SystemOrganization addCategory: #'XML-Writer-Core'!

!String methodsFor: '*XML-Writer-Core' stamp: 'monty 4/20/2016 07:47'!
embedInXMLMarkupWriter: aMarkupWriter
	aMarkupWriter writer string: self! !

!String methodsFor: '*XML-Writer-Core' stamp: 'monty 11/28/2016 03:32'!
xmlWriterLocalNameAfterPrefix: aPrefix
	"optimized"
	| prefixSize |

	(prefixSize := aPrefix size) > 0
		ifTrue: [^ self copyFrom: prefixSize + 2 to: self size]
		ifFalse: [^ self]! !

!String methodsFor: '*XML-Writer-Core' stamp: 'monty 11/28/2016 03:39'!
xmlWriterPrefixBeforeLocalName
	"optimized"
	| i |

	i := self size.
	[i > 1]
		whileTrue: [
			((self at: i) == $:)
				ifTrue: [^ (self copyFrom: 1 to: i - 1)].
			i := i - 1].
	^ ''.! !

!OrderPreservingStringDictionary methodsFor: '*XML-Writer-Core' stamp: 'monty 9/18/2015 21:51'!
writeAttributesOn: aWriter
	self keysDo: [:attributeName |
		aWriter
			writeAttributeName: attributeName
			value: (self at: attributeName ifAbsent: '')]! !

!OrderPreservingStringDictionary methodsFor: '*XML-Writer-Core' stamp: 'monty 11/28/2016 07:16'!
writeCanonicalAttributesOn: aWriter
	| sortedAttributeNames |

	sortedAttributeNames :=
		self keys sort: [:a :b |
			| prefixOfA prefixOfB namespaceURIOfA namespaceURIOfB |
			"the namespace URI is the primary key and
			the local name is the secondary key"
			prefixOfA := a xmlPrefixBeforeLocalName.
			prefixOfB := b xmlPrefixBeforeLocalName.
			namespaceURIOfA := aWriter resolveAttributePrefix: prefixOfA.
			namespaceURIOfB := aWriter resolveAttributePrefix: prefixOfB.
			namespaceURIOfA < namespaceURIOfB
				or: [namespaceURIOfA = namespaceURIOfB
					and: [
						(a xmlLocalNameAfterPrefix: prefixOfA)
							<= (b xmlLocalNameAfterPrefix: prefixOfB)]]].

	"use to:do: for fast writing"
	1 to: sortedAttributeNames size do: [:i | | attributeName |
		attributeName := sortedAttributeNames at: i.
		aWriter
			writeAttributeName: attributeName
			value: (self at: attributeName)].! !

!UndefinedObject methodsFor: '*XML-Writer-Core' stamp: 'monty 4/20/2016 07:47'!
embedInXMLMarkupWriter: aMarkupWriter! !

!Collection methodsFor: '*XML-Writer-Core' stamp: 'monty 4/20/2016 07:47'!
embedInXMLMarkupWriter: aMarkupWriter
	self do: [:each | each embedInXMLMarkupWriter: aMarkupWriter]! !

!Character methodsFor: '*XML-Writer-Core' stamp: 'monty 4/20/2016 07:47'!
embedInXMLMarkupWriter: aMarkupWriter
	self asString embedInXMLMarkupWriter: aMarkupWriter! !

Object subclass: #XMLExternalIDWriter
	instanceVariableNames: 'publicID systemID'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Writer-Core'!

!XMLExternalIDWriter commentStamp: 'monty 11/21/2016 03:56' prior: 0!
This class writes the external IDs for doctype declarations and markup declarations, either as "PUBLIC " followed by the public and system ID literals separated by spaces or "SYSTEM " followed by the system ID literal.!

!XMLExternalIDWriter methodsFor: 'testing' stamp: 'monty 11/21/2016 02:00'!
hasPublicID
	^ self publicID notEmpty! !

!XMLExternalIDWriter methodsFor: 'testing' stamp: 'monty 11/21/2016 02:00'!
hasSystemID
	^ self systemID notEmpty! !

!XMLExternalIDWriter methodsFor: 'testing' stamp: 'monty 11/21/2016 02:00'!
isOmittable
	^ self hasPublicID not
		and: [self hasSystemID not]! !

!XMLExternalIDWriter methodsFor: 'accessing' stamp: 'monty 3/21/2015 22:53'!
publicID
	^ publicID ifNil: [publicID := '']! !

!XMLExternalIDWriter methodsFor: 'accessing' stamp: 'monty 3/21/2015 22:53'!
publicID: aPublicID
	publicID := aPublicID! !

!XMLExternalIDWriter methodsFor: 'accessing' stamp: 'monty 3/21/2015 22:54'!
systemID
	^ systemID ifNil: [systemID := '']! !

!XMLExternalIDWriter methodsFor: 'accessing' stamp: 'monty 3/21/2015 22:54'!
systemID: aSystemID
	systemID := aSystemID! !

!XMLExternalIDWriter methodsFor: 'writing' stamp: 'monty 11/21/2016 02:06'!
writeOn: aWriter
	self hasPublicID
		ifTrue: [self writePublicExternalIDOn: aWriter]
		ifFalse: [
			self hasSystemID
				ifTrue: [self writeSystemExternalIDOn: aWriter]]! !

!XMLExternalIDWriter methodsFor: 'writing' stamp: 'monty 11/21/2016 02:06'!
writePublicExternalIDOn: aWriter
	aWriter
		nextPutAll: 'PUBLIC';
		space;
		writeUnescapedString: self publicID;
		space;
		writeUnescapedString: self systemID! !

!XMLExternalIDWriter methodsFor: 'writing' stamp: 'monty 11/21/2016 02:06'!
writeSystemExternalIDOn: aWriter
	aWriter
		nextPutAll: 'SYSTEM';
		space;
		writeUnescapedString: self systemID! !

XMLExternalIDWriter subclass: #XMLNotationExternalIDWriter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Writer-Core'!

!XMLNotationExternalIDWriter commentStamp: 'monty 11/21/2016 03:55' prior: 0!
This class writes the external ID of a notaion. It omits empty system ID literals for "PUBLIC" external notation IDs.!

!XMLNotationExternalIDWriter methodsFor: 'writing' stamp: 'monty 11/21/2016 02:07'!
writePublicExternalIDOn: aWriter
	aWriter
		nextPutAll: 'PUBLIC';
		space;
		writeUnescapedString: self publicID.
	self hasSystemID
		ifTrue: [
			aWriter
				space;
				writeUnescapedString: self systemID].! !

Object subclass: #XMLMarkupWriter
	instanceVariableNames: 'writer hasStartedWriting'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Writer-Core'!

!XMLMarkupWriter commentStamp: 'JAAyer 7/12/2010 02:41' prior: 0!
This is a base class for markup writers. Subclasses override messages like #writePrologue and #writeEpilogue to write markup using the supplied XMLWriter object.!

XMLMarkupWriter subclass: #XMLCDataWriter
	instanceVariableNames: 'string'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Writer-Core'!

!XMLCDataWriter commentStamp: 'monty 4/20/2016 07:30' prior: 0!
This class writes <!![CDATA[...]]> sections. If canonical XML is enabled, it writes its contents as a regular XML-escaped string instead.!

!XMLCDataWriter methodsFor: 'accessing' stamp: 'monty 12/2/2014 00:22'!
string
	^ string ifNil: [string := '']! !

!XMLCDataWriter methodsFor: 'accessing' stamp: 'monty 12/1/2014 03:55'!
string: aString
	string := aString! !

!XMLCDataWriter methodsFor: 'private' stamp: 'monty 12/1/2014 03:59'!
writeBody
	self writer isCanonical
		ifTrue: [
			self parentMarkupWriter
				ifNotNil: [:parent | parent newStringMarkupBeingWritten].
			self writer writeEscapedContent: self string]
		ifFalse: [self writer nextPutAll: self string]! !

!XMLCDataWriter methodsFor: 'private' stamp: 'monty 11/8/2016 10:45'!
writeEpilogue
	self isCanonical
		ifFalse: [
			self writer
				nextPutAll: ']]>';
				formatAfterCData: self]! !

!XMLCDataWriter methodsFor: 'private' stamp: 'monty 11/8/2016 10:45'!
writePrologue
	self isCanonical
		ifFalse: [
			self writer
				formatBeforeCData: self;
				nextPutAll: '<!![CDATA[']! !

XMLMarkupWriter subclass: #XMLCommentWriter
	instanceVariableNames: 'string'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Writer-Core'!

!XMLCommentWriter commentStamp: 'monty 4/17/2016 07:12' prior: 0!
This class writes XML <!!-- ... --> comments, but not in DTD internal subsets if canonical XML is enabled.!

!XMLCommentWriter methodsFor: 'testing' stamp: 'monty 11/21/2016 00:48'!
isOmittable
	^ self isCanonical
		and: [self isInDoctypeDeclaration]! !

!XMLCommentWriter methodsFor: 'accessing' stamp: 'monty 12/2/2014 00:22'!
string
	^ string ifNil: [string := '']! !

!XMLCommentWriter methodsFor: 'accessing' stamp: 'monty 12/1/2014 03:56'!
string: aString
	string := aString! !

!XMLCommentWriter methodsFor: 'private' stamp: 'monty 11/21/2016 00:44'!
writeBody
	self isOmittable
		ifFalse: [self writer nextPutAll: self string]! !

!XMLCommentWriter methodsFor: 'private' stamp: 'monty 11/21/2016 00:44'!
writeEpilogue
	self isOmittable
		ifFalse: [
			self writer
				nextPutAll: '-->';
				formatAfterComment: self]! !

!XMLCommentWriter methodsFor: 'private' stamp: 'monty 11/21/2016 00:45'!
writePrologue
	self isOmittable
		ifFalse: [
			self writer
				formatBeforeComment: self;
				nextPutAll: '<!!--']! !

XMLMarkupWriter subclass: #XMLEmbeddedMarkupWriter
	instanceVariableNames: 'hasEmbeddedStringMarkup hasEmbeddedMarkupWriters'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Writer-Core'!

!XMLEmbeddedMarkupWriter commentStamp: 'monty 6/15/2015 02:43' prior: 0!
This is an abstract class for markup writers that support embedding markup using #with:.!

XMLEmbeddedMarkupWriter subclass: #XMLDoctypeDeclarationWriter
	instanceVariableNames: 'root externalIDWriter'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Writer-Core'!

!XMLDoctypeDeclarationWriter commentStamp: 'monty 6/15/2015 02:32' prior: 0!
This class writes <!!DOCTYPE ...> declarations. You can set the root element and the public and system IDs using the messages under "accessing" before specifying any internal subset with #with:.!

!XMLDoctypeDeclarationWriter methodsFor: 'accessing' stamp: 'monty 3/21/2015 22:54'!
externalIDWriter
	^ externalIDWriter ifNil: [externalIDWriter := XMLExternalIDWriter new]! !

!XMLDoctypeDeclarationWriter methodsFor: 'testing' stamp: 'monty 11/9/2016 11:59'!
isDoctypeDeclarationWriter
	^ true! !

!XMLDoctypeDeclarationWriter methodsFor: 'testing' stamp: 'monty 11/21/2016 01:37'!
isExternalIDOmittable
	^ self externalIDWriter isOmittable! !

!XMLDoctypeDeclarationWriter methodsFor: 'testing' stamp: 'monty 11/21/2016 00:49'!
isOmittable
	^ self isCanonical! !

!XMLDoctypeDeclarationWriter methodsFor: 'accessing' stamp: 'JAAyer 7/27/2010 08:20'!
publicID
	^ self externalIDWriter publicID! !

!XMLDoctypeDeclarationWriter methodsFor: 'accessing' stamp: 'monty 3/21/2015 22:47'!
publicID: aPublicID
	self externalIDWriter publicID: aPublicID! !

!XMLDoctypeDeclarationWriter methodsFor: 'accessing' stamp: 'JAAyer 7/26/2010 14:52'!
root
	^ root ifNil: ['']! !

!XMLDoctypeDeclarationWriter methodsFor: 'accessing' stamp: 'JAAyer 7/26/2010 14:52'!
root: aRootElementName
	root := aRootElementName! !

!XMLDoctypeDeclarationWriter methodsFor: 'accessing' stamp: 'JAAyer 7/27/2010 08:20'!
systemID
	^ self externalIDWriter systemID! !

!XMLDoctypeDeclarationWriter methodsFor: 'accessing' stamp: 'monty 3/21/2015 22:47'!
systemID: aSystemID
	self externalIDWriter systemID: aSystemID! !

!XMLDoctypeDeclarationWriter methodsFor: 'writing' stamp: 'monty 6/15/2015 01:10'!
withInternalSubset: aStringOrCollectionOrBlock
	self deprecated: 'use #with: instead'.

	self with: aStringOrCollectionOrBlock.! !

!XMLDoctypeDeclarationWriter methodsFor: 'private' stamp: 'monty 11/21/2016 00:44'!
writeBody: aStringOrCollectionOrBlock
	self isOmittable
		ifFalse: [super writeBody: aStringOrCollectionOrBlock]! !

!XMLDoctypeDeclarationWriter methodsFor: 'private' stamp: 'monty 11/21/2016 00:44'!
writeEmbeddedMarkupPrologue
	self isOmittable
		ifFalse: [
			self writer
				space;
				nextPut: $[;
				formatBeforeInternalSubset: self]! !

!XMLDoctypeDeclarationWriter methodsFor: 'private' stamp: 'monty 11/21/2016 00:44'!
writeEpilogue
	self isOmittable
		ifTrue: [^ self].

	self hasEmbeddedMarkup
		ifTrue: [
			self writer
				formatAfterInternalSubset: self;
				nextPut: $]].

	self writer
		nextPut: $>;
		formatAfterDoctypeDeclaration: self.! !

!XMLDoctypeDeclarationWriter methodsFor: 'private' stamp: 'monty 11/21/2016 01:23'!
writePrologue
	self isOmittable
		ifTrue: [^ self].

	self writer
		formatBeforeDoctypeDeclaration: self;
		nextPutAll: '<!!DOCTYPE';
		space;
		writeName: self root.

	self isExternalIDOmittable
		ifFalse: [
			self writer space.
			self externalIDWriter writeOn: self writer].! !

!XMLEmbeddedMarkupWriter methodsFor: 'testing' stamp: 'monty 12/1/2014 04:41'!
hasEmbeddedMarkup
	^ hasEmbeddedStringMarkup | hasEmbeddedMarkupWriters! !

!XMLEmbeddedMarkupWriter methodsFor: 'testing' stamp: 'monty 11/30/2014 06:17'!
hasEmbeddedMarkupWriters
	^ hasEmbeddedMarkupWriters! !

!XMLEmbeddedMarkupWriter methodsFor: 'testing' stamp: 'monty 11/30/2014 06:19'!
hasEmbeddedStringMarkup
	^ hasEmbeddedStringMarkup! !

!XMLEmbeddedMarkupWriter methodsFor: 'initialization' stamp: 'monty 9/20/2015 23:25'!
initialize
	super initialize.

	hasEmbeddedStringMarkup := false.
	hasEmbeddedMarkupWriters := false.! !

!XMLEmbeddedMarkupWriter methodsFor: 'private' stamp: 'monty 2/26/2016 09:10'!
newMarkupWriterBeingPushed
	self hasStartedWriting
		ifTrue: [
			self hasEmbeddedMarkup
				ifFalse: [self writeEmbeddedMarkupPrologue].
			hasEmbeddedMarkupWriters := true]
		ifFalse: [self write]! !

!XMLEmbeddedMarkupWriter methodsFor: 'private' stamp: 'monty 2/26/2016 09:10'!
newStringMarkupBeingWritten
	self hasStartedWriting
		ifTrue: [
			self hasEmbeddedMarkup
				ifFalse: [self writeEmbeddedMarkupPrologue].
			hasEmbeddedStringMarkup := true]
		ifFalse: [self write]! !

!XMLEmbeddedMarkupWriter methodsFor: 'writing' stamp: 'monty 2/26/2016 09:12'!
with: aStringOrCollectionOrBlock
	self hasStartedWriting
		ifFalse: [
			self
				startWriting;
				writeBody: aStringOrCollectionOrBlock;
				finishWriting]! !

!XMLEmbeddedMarkupWriter methodsFor: 'private' stamp: 'monty 10/12/2014 19:21'!
writeBody
	self writeBody: nil! !

!XMLEmbeddedMarkupWriter methodsFor: 'private' stamp: 'monty 4/20/2016 07:47'!
writeBody: aStringOrCollectionOrBlock
	aStringOrCollectionOrBlock embedInXMLMarkupWriter: self! !

!XMLEmbeddedMarkupWriter methodsFor: 'private' stamp: 'monty 1/13/2014 23:27'!
writeEmbeddedMarkupPrologue! !

XMLEmbeddedMarkupWriter subclass: #XMLTagWriter
	instanceVariableNames: 'name xmlnsDeclarations attributes'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Writer-Core'!

!XMLTagWriter commentStamp: 'monty 4/17/2016 07:18' prior: 0!
This class writes XML tags. You can specifiy the attributes of the start tag using the #attribute* messages, and you can add xmlns: namespace declarations using the #xmlns* messages.

Empty elements will generate self-closed tags like "<empty/>" unless isCanonical is enabled or outputsSelfClosingTags is disabled.

Redundant namespace declarations won't be generated unless omitsRedundantXMLNSDeclarations is disabled.!

!XMLTagWriter methodsFor: 'accessing' stamp: 'JAAyer 8/9/2010 14:54'!
attributeAt: aName
	^ self attributeAt: aName ifAbsent: ['']! !

!XMLTagWriter methodsFor: 'accessing' stamp: 'JAAyer 7/26/2010 12:38'!
attributeAt: aName ifAbsent: aBlock
	^ self attributes at: aName ifAbsent: aBlock! !

!XMLTagWriter methodsFor: 'accessing' stamp: 'JAAyer 7/26/2010 12:38'!
attributeAt: aName ifAbsentPut: aBlock
	^ self attributes at: aName ifAbsentPut: aBlock! !

!XMLTagWriter methodsFor: 'accessing' stamp: 'JAAyer 7/26/2010 12:38'!
attributeAt: aName put: aValue
	^ self attributes at: aName put: aValue! !

!XMLTagWriter methodsFor: 'accessing' stamp: 'monty 11/20/2014 19:48'!
attributes
	^ attributes ifNil: [attributes := OrderPreservingStringDictionary new]! !

!XMLTagWriter methodsFor: 'accessing' stamp: 'monty 11/20/2014 19:48'!
attributes: aDictionaryOrArray
	attributes := OrderPreservingStringDictionary newFrom: aDictionaryOrArray! !

!XMLTagWriter methodsFor: 'testing' stamp: 'monty 9/19/2015 02:13'!
hasAttributes
	^ attributes notNil and: [attributes notEmpty]! !

!XMLTagWriter methodsFor: 'testing' stamp: 'monty 9/18/2015 21:48'!
hasXMLNSDeclarations
	^ xmlnsDeclarations notNil and: [xmlnsDeclarations notEmpty]! !

!XMLTagWriter methodsFor: 'testing' stamp: 'monty 10/6/2014 23:30'!
includesAttribute: anAttribute
	^ self attributes includesKey: anAttribute! !

!XMLTagWriter methodsFor: 'testing' stamp: 'monty 9/18/2015 21:28'!
includesXMLNS: aPrefix
	^ self xmlnsDeclarations includesKey: aPrefix! !

!XMLTagWriter methodsFor: 'testing' stamp: 'monty 10/12/2014 21:12'!
isTagWriter
	^ true! !

!XMLTagWriter methodsFor: 'accessing' stamp: 'monty 9/18/2015 21:07'!
name
	^ name ifNil: [name := '']! !

!XMLTagWriter methodsFor: 'accessing' stamp: 'JAAyer 6/20/2010 16:43'!
name: aName
	name := aName! !

!XMLTagWriter methodsFor: 'private' stamp: 'monty 3/24/2015 19:26'!
setAttributes: aDictionary
	attributes := aDictionary! !

!XMLTagWriter methodsFor: 'private' stamp: 'monty 3/24/2015 19:26'!
setXMLNSDeclarations: aDictionary 
	xmlnsDeclarations := aDictionary! !

!XMLTagWriter methodsFor: 'private' stamp: 'monty 12/1/2014 23:36'!
writeEmbeddedMarkupPrologue
	self writer
		nextPut: $>;
		formatAfterStartTag: self! !

!XMLTagWriter methodsFor: 'private' stamp: 'monty 9/18/2015 22:28'!
writeEpilogue
	(self hasEmbeddedMarkup
		or: [self writer isCanonical
			or: [self writer outputsSelfClosingTags not]])
		ifTrue: [
			self hasEmbeddedMarkup
				ifFalse: [
					self writer
						nextPut: $>;
						formatAfterStartTag: self].
			self writer
				formatBeforeEndTag: self;
				nextPut: $<;
				nextPut: $/;
				writeName: self name;
				nextPut: $>;
				formatAfterEndTag: self]
		ifFalse: [
			self writer
				space;
				nextPut: $/;
				nextPut: $>;
				formatAfterEndTag: self]! !

!XMLTagWriter methodsFor: 'private' stamp: 'monty 9/18/2015 21:56'!
writePrologue
	self writer
		formatBeforeStartTag: self;
		nextPut: $<;
		writeName: self name;
		writeXMLNSDeclarations: xmlnsDeclarations;
		writeAttributes: attributes! !

!XMLTagWriter methodsFor: 'accessing' stamp: 'monty 10/6/2014 20:59'!
xmlns
	"default namespace"
	^ self xmlnsDeclarations at: ''! !

!XMLTagWriter methodsFor: 'accessing' stamp: 'monty 3/21/2015 22:47'!
xmlns: aURI
	"set default namespace"
	self xmlnsDeclarations at: '' put: aURI! !

!XMLTagWriter methodsFor: 'accessing' stamp: 'monty 10/6/2014 19:47'!
xmlnsAt: aPrefix
	^ self xmlnsDeclarations at: aPrefix ifAbsent: ['']! !

!XMLTagWriter methodsFor: 'accessing' stamp: 'monty 10/6/2014 19:47'!
xmlnsAt: aPrefix ifAbsent: aBlock
	^ self xmlnsDeclarations at: aPrefix ifAbsent: aBlock! !

!XMLTagWriter methodsFor: 'accessing' stamp: 'monty 9/5/2015 08:42'!
xmlnsAt: aPrefix ifAbsentPut: aBlock
	^ self xmlnsDeclarations at: aPrefix ifAbsentPut: aBlock! !

!XMLTagWriter methodsFor: 'accessing' stamp: 'monty 10/6/2014 19:47'!
xmlnsAt: aPrefix put: aValue
	^ self xmlnsDeclarations at: aPrefix put: aValue! !

!XMLTagWriter methodsFor: 'accessing' stamp: 'monty 3/21/2015 22:55'!
xmlnsDeclarations
	^ xmlnsDeclarations
		ifNil: [xmlnsDeclarations := OrderPreservingStringDictionary new: 5]! !

!XMLTagWriter methodsFor: 'accessing' stamp: 'monty 11/20/2014 19:48'!
xmlnsDeclarations: aDictionaryOrArray
	"set xmlns decls using a dictionary or array of prefix/uri associations.
	empty prefix is default xmlns."
	xmlnsDeclarations := OrderPreservingStringDictionary newFrom: aDictionaryOrArray! !

XMLMarkupWriter subclass: #XMLEntityReferenceWriter
	instanceVariableNames: 'name isParameter'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Writer-Core'!

!XMLEntityReferenceWriter commentStamp: 'monty 4/17/2016 07:13' prior: 0!
This class writes general and parameter entity references.!

!XMLEntityReferenceWriter methodsFor: 'testing' stamp: 'monty 12/2/2014 00:33'!
isParameter
	^ isParameter ifNil: [isParameter := false]! !

!XMLEntityReferenceWriter methodsFor: 'accessing' stamp: 'monty 12/2/2014 00:32'!
isParameter: aBoolean
	isParameter := aBoolean! !

!XMLEntityReferenceWriter methodsFor: 'accessing' stamp: 'monty 12/2/2014 00:22'!
name
	^ name ifNil: [name := '']! !

!XMLEntityReferenceWriter methodsFor: 'accessing' stamp: 'monty 12/2/2014 00:42'!
name: aName
	name := aName! !

!XMLEntityReferenceWriter methodsFor: 'private' stamp: 'monty 12/2/2014 00:40'!
writeBody
	self writer writeName: self name! !

!XMLEntityReferenceWriter methodsFor: 'private' stamp: 'monty 5/8/2015 04:16'!
writeEpilogue
	self writer
		nextPut: $;;
		formatAfterEntityReference: self! !

!XMLEntityReferenceWriter methodsFor: 'private' stamp: 'monty 12/2/2014 00:34'!
writePrologue
	self writer
		formatBeforeEntityReference: self;
		nextPut:
			(self isParameter
				ifTrue: [$%]
				ifFalse: [$&])! !

XMLMarkupWriter subclass: #XMLMarkupDeclarationWriter
	instanceVariableNames: 'name'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Writer-Core'!

!XMLMarkupDeclarationWriter commentStamp: 'monty 4/17/2016 07:16' prior: 0!
This is an abstract class for DTD markup declaration writers.!

XMLMarkupDeclarationWriter subclass: #XMLAttributeDeclarationWriter
	instanceVariableNames: 'attribute type defaultPragma defaultValue'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Writer-Core'!

!XMLAttributeDeclarationWriter commentStamp: 'monty 4/17/2016 07:15' prior: 0!
This class writes attribute <!!ATTLIST ...> declarations for DTD internal subsets, but not if canonical XML is enabled.!

!XMLAttributeDeclarationWriter methodsFor: 'accessing' stamp: 'monty 12/21/2013 02:55'!
attribute
	^ attribute ifNil: [attribute := '']! !

!XMLAttributeDeclarationWriter methodsFor: 'accessing' stamp: 'monty 12/21/2013 02:55'!
attribute: aName
	attribute := aName! !

!XMLAttributeDeclarationWriter methodsFor: 'accessing' stamp: 'JAAyer 7/27/2010 08:27'!
declarationType
	^ 'ATTLIST'! !

!XMLAttributeDeclarationWriter methodsFor: 'accessing' stamp: 'monty 12/31/2013 00:15'!
defaultPragma
	^ defaultPragma ifNil: [defaultPragma := '']! !

!XMLAttributeDeclarationWriter methodsFor: 'accessing' stamp: 'monty 12/31/2013 00:17'!
defaultPragma: aDefaultPragma
	defaultPragma := aDefaultPragma! !

!XMLAttributeDeclarationWriter methodsFor: 'accessing' stamp: 'monty 12/31/2013 00:15'!
defaultValue
	^ defaultValue ifNil: [defaultValue := '']! !

!XMLAttributeDeclarationWriter methodsFor: 'accessing' stamp: 'monty 12/31/2013 00:17'!
defaultValue: aDefaultValue
	defaultValue := aDefaultValue! !

!XMLAttributeDeclarationWriter methodsFor: 'accessing' stamp: 'monty 12/21/2013 02:54'!
element
	^ self name! !

!XMLAttributeDeclarationWriter methodsFor: 'accessing' stamp: 'monty 12/21/2013 02:54'!
element: aName
	self name: aName! !

!XMLAttributeDeclarationWriter methodsFor: 'accessing' stamp: 'JAAyer 8/7/2010 13:51'!
type
	^ type ifNil: [type := 'CDATA']! !

!XMLAttributeDeclarationWriter methodsFor: 'accessing' stamp: 'JAAyer 7/27/2010 08:30'!
type: aType
	type := aType! !

!XMLAttributeDeclarationWriter methodsFor: 'private' stamp: 'monty 11/21/2016 00:44'!
writeBody
	self isOmittable
		ifTrue: [^ self].

	self writer
		writeName: self attribute;
		space;
		writeName: self type.

	self
		writeDefaultPragma;
		writeDefaultValue.! !

!XMLAttributeDeclarationWriter methodsFor: 'private' stamp: 'monty 10/12/2014 20:46'!
writeDefaultPragma
	self defaultPragma
		ifNotEmpty: [:pragma |
			self writer space.

			(pragma first == $#)
				ifFalse: [self writer nextPut: $#].

			self writer nextPutAll: pragma]! !

!XMLAttributeDeclarationWriter methodsFor: 'private' stamp: 'monty 10/12/2014 20:46'!
writeDefaultValue
	self defaultValue
		ifNotEmpty: [:value |
			self writer
				space;
				writeEscapedContentString: value]! !

XMLMarkupDeclarationWriter subclass: #XMLElementDeclarationWriter
	instanceVariableNames: 'contentModel'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Writer-Core'!

!XMLElementDeclarationWriter commentStamp: 'monty 4/17/2016 07:15' prior: 0!
This class writes <!!ELEMENT ...> declarations for DTD internal subsets, but not if canonical XML is enabled.!

!XMLElementDeclarationWriter methodsFor: 'accessing' stamp: 'JAAyer 8/7/2010 13:52'!
contentModel
	^ contentModel ifNil: [contentModel := '']! !

!XMLElementDeclarationWriter methodsFor: 'accessing' stamp: 'JAAyer 7/26/2010 15:11'!
contentModel: aContentModelString
	contentModel := aContentModelString! !

!XMLElementDeclarationWriter methodsFor: 'accessing' stamp: 'JAAyer 7/27/2010 08:27'!
declarationType
	^ 'ELEMENT'! !

!XMLElementDeclarationWriter methodsFor: 'accessing' stamp: 'monty 12/21/2013 03:17'!
element
	^ self name! !

!XMLElementDeclarationWriter methodsFor: 'accessing' stamp: 'monty 12/21/2013 03:17'!
element: aName
	self name: aName! !

!XMLElementDeclarationWriter methodsFor: 'private' stamp: 'monty 11/21/2016 00:44'!
writeBody
	self isOmittable
		ifFalse: [self writer nextPutAll: self contentModel]! !

XMLMarkupDeclarationWriter subclass: #XMLEntityDeclarationWriter
	instanceVariableNames: 'isParameter replacement externalIDWriter ndata'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Writer-Core'!

!XMLEntityDeclarationWriter commentStamp: 'monty 4/17/2016 07:16' prior: 0!
This class writes general and parameter <!!ENTITY ...> declarations for DTD internal subsets, but not if canonical XML is enabled.!

!XMLEntityDeclarationWriter methodsFor: 'accessing' stamp: 'monty 11/20/2013 23:41'!
declarationType
	^ self isParameter
		ifTrue: ['ENTITY %']
		ifFalse: ['ENTITY']! !

!XMLEntityDeclarationWriter methodsFor: 'accessing' stamp: 'monty 3/21/2015 22:56'!
externalIDWriter
	^ externalIDWriter ifNil: [externalIDWriter := XMLExternalIDWriter new]! !

!XMLEntityDeclarationWriter methodsFor: 'testing' stamp: 'monty 11/13/2013 19:36'!
isParameter
	^ isParameter ifNil: [isParameter := false]! !

!XMLEntityDeclarationWriter methodsFor: 'accessing' stamp: 'monty 11/13/2013 19:36'!
isParameter: aBoolean
	isParameter := aBoolean! !

!XMLEntityDeclarationWriter methodsFor: 'testing' stamp: 'monty 11/21/2016 00:53'!
isUnparsed
	^ self ndata notEmpty! !

!XMLEntityDeclarationWriter methodsFor: 'accessing' stamp: 'JAAyer 8/7/2010 14:45'!
ndata
	^ ndata ifNil: [ndata := '']! !

!XMLEntityDeclarationWriter methodsFor: 'accessing' stamp: 'JAAyer 7/26/2010 17:37'!
ndata: aNotationName
	ndata := aNotationName! !

!XMLEntityDeclarationWriter methodsFor: 'accessing' stamp: 'JAAyer 7/27/2010 08:20'!
publicID
	^ self externalIDWriter publicID! !

!XMLEntityDeclarationWriter methodsFor: 'accessing' stamp: 'monty 3/21/2015 22:47'!
publicID: aPublicID
	self externalIDWriter publicID: aPublicID! !

!XMLEntityDeclarationWriter methodsFor: 'accessing' stamp: 'JAAyer 8/7/2010 14:44'!
replacement
	^ replacement ifNil: [replacement := '']! !

!XMLEntityDeclarationWriter methodsFor: 'accessing' stamp: 'JAAyer 7/26/2010 17:35'!
replacement: aReplacement
	replacement := aReplacement! !

!XMLEntityDeclarationWriter methodsFor: 'accessing' stamp: 'JAAyer 7/27/2010 08:20'!
systemID
	^ self externalIDWriter systemID! !

!XMLEntityDeclarationWriter methodsFor: 'accessing' stamp: 'monty 3/21/2015 22:47'!
systemID: aSystemID
	self externalIDWriter systemID: aSystemID! !

!XMLEntityDeclarationWriter methodsFor: 'private' stamp: 'monty 11/27/2016 02:36'!
writeBody
	self isOmittable
		ifTrue: [^ self].

	self externalIDWriter isOmittable
		ifTrue: [
			self writer writeEscapedEntityValueString:
				self replacement]
		ifFalse: [
			self externalIDWriter writeOn: self writer.
			self isUnparsed
				ifTrue: [
					self writer
						space;
						nextPutAll: 'NDATA';
						space;
						writeName: self ndata]].! !

!XMLMarkupDeclarationWriter methodsFor: 'accessing' stamp: 'JAAyer 7/27/2010 08:26'!
declarationType
	self subclassResponsibility! !

!XMLMarkupDeclarationWriter methodsFor: 'testing' stamp: 'monty 11/21/2016 00:43'!
isOmittable
	^ self isCanonical! !

!XMLMarkupDeclarationWriter methodsFor: 'accessing' stamp: 'JAAyer 7/26/2010 14:43'!
name
	^ name ifNil: ['']! !

!XMLMarkupDeclarationWriter methodsFor: 'accessing' stamp: 'JAAyer 6/20/2010 17:02'!
name: aName
	name := aName! !

!XMLMarkupDeclarationWriter methodsFor: 'private' stamp: 'monty 11/21/2016 00:43'!
writeEpilogue
	self isOmittable
		ifFalse: [
			self writer
				nextPut: $>;
				formatAfterDeclaration: self]! !

!XMLMarkupDeclarationWriter methodsFor: 'private' stamp: 'monty 11/21/2016 00:44'!
writePrologue
	self isOmittable
		ifFalse: [
			self writer
				formatBeforeDeclaration: self;
				nextPutAll: '<!!';
				writeName: self declarationType;
				space;
				writeName: self name;
				space]! !

XMLMarkupDeclarationWriter subclass: #XMLNotationDeclarationWriter
	instanceVariableNames: 'externalIDWriter'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Writer-Core'!

!XMLNotationDeclarationWriter commentStamp: 'monty 4/17/2016 07:15' prior: 0!
This class writes <!!NOTATION ...> declarations for DTD internal subsets, but not if canonical XML is enabled.!

!XMLNotationDeclarationWriter methodsFor: 'accessing' stamp: 'JAAyer 7/27/2010 08:27'!
declarationType
	^ 'NOTATION'! !

!XMLNotationDeclarationWriter methodsFor: 'accessing' stamp: 'monty 11/21/2016 02:07'!
externalIDWriter
	^ externalIDWriter ifNil: [externalIDWriter := XMLNotationExternalIDWriter new]! !

!XMLNotationDeclarationWriter methodsFor: 'accessing' stamp: 'JAAyer 7/27/2010 08:20'!
publicID
	^ self externalIDWriter publicID! !

!XMLNotationDeclarationWriter methodsFor: 'accessing' stamp: 'monty 3/22/2015 02:19'!
publicID: aPublicID
	self externalIDWriter publicID: aPublicID! !

!XMLNotationDeclarationWriter methodsFor: 'accessing' stamp: 'JAAyer 7/27/2010 08:20'!
systemID
	^ self externalIDWriter systemID! !

!XMLNotationDeclarationWriter methodsFor: 'accessing' stamp: 'monty 3/21/2015 22:47'!
systemID: aSystemID
	self externalIDWriter systemID: aSystemID! !

!XMLNotationDeclarationWriter methodsFor: 'private' stamp: 'monty 11/21/2016 00:44'!
writeBody
	self isOmittable
		ifFalse: [self externalIDWriter writeOn: self writer]! !

!XMLMarkupWriter class methodsFor: 'instance creation' stamp: 'monty 9/20/2015 23:24'!
writer: aWriter
	^ self new writer: aWriter! !

!XMLMarkupWriter methodsFor: 'converting' stamp: 'monty 9/19/2015 21:59'!
asString
	^ self writer asString! !

!XMLMarkupWriter methodsFor: 'closing' stamp: 'monty 5/6/2015 16:31'!
close
	self writer close! !

!XMLMarkupWriter methodsFor: 'accessing' stamp: 'monty 5/6/2015 16:31'!
contents
	^ self writer contents! !

!XMLMarkupWriter methodsFor: 'writing' stamp: 'monty 2/26/2016 09:09'!
finishWriting
	self hasEmbeddedMarkupWriters
		ifTrue: [self writer writeUnwrittenMarkupWritersEmbeddedIn: self].
	self writeEpilogue.

	self writer popActiveMarkupWriter.! !

!XMLMarkupWriter methodsFor: 'testing' stamp: 'monty 1/14/2014 00:18'!
hasEmbeddedMarkup
	^ false! !

!XMLMarkupWriter methodsFor: 'testing' stamp: 'monty 10/4/2014 04:21'!
hasEmbeddedMarkupWriters
	^ false! !

!XMLMarkupWriter methodsFor: 'testing' stamp: 'monty 12/1/2014 05:19'!
hasEmbeddedStringMarkup
	^ false! !

!XMLMarkupWriter methodsFor: 'testing' stamp: 'monty 2/26/2016 09:11'!
hasStartedWriting
	^ hasStartedWriting! !

!XMLMarkupWriter methodsFor: 'initialization' stamp: 'monty 2/26/2016 09:11'!
initialize
	super initialize.

	hasStartedWriting := false.! !

!XMLMarkupWriter methodsFor: 'testing' stamp: 'monty 11/8/2016 10:45'!
isCanonical
	^ self writer isCanonical! !

!XMLMarkupWriter methodsFor: 'testing' stamp: 'monty 11/9/2016 11:59'!
isDoctypeDeclarationWriter
	^ false! !

!XMLMarkupWriter methodsFor: 'testing' stamp: 'monty 2/17/2016 23:48'!
isEmbedded
	^ self parentMarkupWriter notNil! !

!XMLMarkupWriter methodsFor: 'testing' stamp: 'monty 12/1/2014 09:53'!
isEmbeddedWithStringMarkup
	| parent |

	^ (parent := self parentMarkupWriter) notNil
		and: [parent hasEmbeddedStringMarkup]! !

!XMLMarkupWriter methodsFor: 'testing' stamp: 'monty 11/21/2016 00:47'!
isInDoctypeDeclaration
	^ self parentMarkupWriter
		ifNil: [false]
		ifNotNil: [:parent | parent isDoctypeDeclarationWriter]! !

!XMLMarkupWriter methodsFor: 'testing' stamp: 'monty 10/12/2014 21:11'!
isTagWriter
	^ false! !

!XMLMarkupWriter methodsFor: 'testing' stamp: 'monty 9/18/2015 23:21'!
isUnsafeTagWriter
	^ false! !

!XMLMarkupWriter methodsFor: 'private' stamp: 'monty 1/16/2014 23:00'!
newMarkupWriterBeingPushed
	self write! !

!XMLMarkupWriter methodsFor: 'private' stamp: 'monty 11/30/2014 06:21'!
newStringMarkupBeingWritten
	self write! !

!XMLMarkupWriter methodsFor: 'accessing' stamp: 'monty 12/1/2014 04:07'!
parentMarkupWriter
	^ self writer parentMarkupWriterOf: self! !

!XMLMarkupWriter methodsFor: 'printing' stamp: 'JAAyer 7/21/2010 09:26'!
printOn: aStream
	self writer printOn: aStream! !

!XMLMarkupWriter methodsFor: 'writing' stamp: 'monty 2/26/2016 09:11'!
startWriting
	hasStartedWriting := true.
	self writePrologue.! !

!XMLMarkupWriter methodsFor: 'writing' stamp: 'monty 2/26/2016 09:10'!
write
	self hasStartedWriting
		ifFalse: [
			self
				startWriting;
				writeBody;
				finishWriting]! !

!XMLMarkupWriter methodsFor: 'private' stamp: 'JAAyer 7/26/2010 12:12'!
writeBody! !

!XMLMarkupWriter methodsFor: 'private' stamp: 'JAAyer 6/20/2010 12:08'!
writeEpilogue! !

!XMLMarkupWriter methodsFor: 'private' stamp: 'JAAyer 6/20/2010 12:08'!
writePrologue! !

!XMLMarkupWriter methodsFor: 'accessing' stamp: 'JAAyer 6/18/2010 22:05'!
writer
	^ writer! !

!XMLMarkupWriter methodsFor: 'accessing' stamp: 'monty 9/21/2015 01:01'!
writer: aWriter
	writer := aWriter! !

XMLMarkupWriter subclass: #XMLPIWriter
	instanceVariableNames: 'target data'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Writer-Core'!

!XMLPIWriter commentStamp: 'monty 4/17/2016 07:17' prior: 0!
This class writes <?...?> processing instructions, but not in DTD internal subsets if canonical XML is enabled.!

!XMLPIWriter methodsFor: 'accessing' stamp: 'JAAyer 7/26/2010 14:10'!
data
	^ data ifNil: ['']! !

!XMLPIWriter methodsFor: 'accessing' stamp: 'JAAyer 7/26/2010 12:39'!
data: aString
	data := aString! !

!XMLPIWriter methodsFor: 'testing' stamp: 'monty 11/8/2016 07:56'!
hasData
	^ self data notEmpty! !

!XMLPIWriter methodsFor: 'testing' stamp: 'monty 11/21/2016 00:48'!
isOmittable
	^ self isCanonical
		and: [self isInDoctypeDeclaration]! !

!XMLPIWriter methodsFor: 'accessing' stamp: 'JAAyer 7/26/2010 14:10'!
target
	^ target ifNil: ['']! !

!XMLPIWriter methodsFor: 'accessing' stamp: 'monty 3/21/2015 22:50'!
target: aString
	target := aString! !

!XMLPIWriter methodsFor: 'private' stamp: 'monty 11/21/2016 00:45'!
writeBody
	(self isOmittable
		or: [self hasData not])
		ifFalse: [
			self writer
				space;
				nextPutAll: self data]! !

!XMLPIWriter methodsFor: 'private' stamp: 'monty 11/21/2016 00:45'!
writeEpilogue
	self isOmittable
		ifFalse: [
			self writer
				nextPutAll: '?>';
				formatAfterPI: self]! !

!XMLPIWriter methodsFor: 'private' stamp: 'monty 11/21/2016 00:45'!
writePrologue
	self isOmittable
		ifFalse: [
			self writer
				formatBeforePI: self;
				nextPutAll: '<?';
				writeName: self target]! !

XMLMarkupWriter subclass: #XMLUnsafeTagWriter
	instanceVariableNames: 'name'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Writer-Core'!

!XMLUnsafeTagWriter commentStamp: 'monty 9/19/2015 16:15' prior: 0!
This is an abstract class for tag writers that are "unsafe" because they can make non-wellformed XML if they aren't used correctly.!

XMLUnsafeTagWriter subclass: #XMLUnsafeEndTagWriter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Writer-Core'!

!XMLUnsafeEndTagWriter commentStamp: 'monty 9/19/2015 16:15' prior: 0!
This class is an end tag writer used to close an element started by an XMLUnsafeStartTagWriter.!

!XMLUnsafeEndTagWriter methodsFor: 'private' stamp: 'monty 9/18/2015 22:07'!
writeBody
	self writer writeName: self name! !

!XMLUnsafeEndTagWriter methodsFor: 'private' stamp: 'monty 9/18/2015 22:07'!
writeEpilogue
	self writer
		nextPut: $>;
		formatAfterEndTag: self! !

!XMLUnsafeEndTagWriter methodsFor: 'private' stamp: 'monty 9/18/2015 22:06'!
writePrologue
	self writer
		formatBeforeEndTag: self;
		nextPut: $<;
		nextPut: $/! !

XMLUnsafeTagWriter subclass: #XMLUnsafeStartTagWriter
	instanceVariableNames: 'xmlnsDeclarations attributes'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Writer-Core'!

!XMLUnsafeStartTagWriter commentStamp: 'monty 9/19/2015 17:05' prior: 0!
This class is a start tag writer. It does not automatically close itself like XMLTagWriter and will make non-wellformed XML if an end tag isn't generated for it.!

!XMLUnsafeStartTagWriter methodsFor: 'accessing' stamp: 'monty 9/18/2015 21:26'!
attributeAt: aName
	^ self attributeAt: aName ifAbsent: ['']! !

!XMLUnsafeStartTagWriter methodsFor: 'accessing' stamp: 'monty 9/18/2015 21:26'!
attributeAt: aName ifAbsent: aBlock
	^ self attributes at: aName ifAbsent: aBlock! !

!XMLUnsafeStartTagWriter methodsFor: 'accessing' stamp: 'monty 9/18/2015 21:26'!
attributeAt: aName ifAbsentPut: aBlock
	^ self attributes at: aName ifAbsentPut: aBlock! !

!XMLUnsafeStartTagWriter methodsFor: 'accessing' stamp: 'monty 9/18/2015 21:26'!
attributeAt: aName put: aValue
	^ self attributes at: aName put: aValue! !

!XMLUnsafeStartTagWriter methodsFor: 'accessing' stamp: 'monty 9/18/2015 21:26'!
attributes
	^ attributes ifNil: [attributes := OrderPreservingStringDictionary new]! !

!XMLUnsafeStartTagWriter methodsFor: 'accessing' stamp: 'monty 9/18/2015 21:26'!
attributes: aDictionaryOrArray
	attributes := OrderPreservingStringDictionary newFrom: aDictionaryOrArray! !

!XMLUnsafeStartTagWriter methodsFor: 'testing' stamp: 'monty 9/19/2015 02:13'!
hasAttributes
	^ attributes notNil and: [attributes notEmpty]! !

!XMLUnsafeStartTagWriter methodsFor: 'testing' stamp: 'monty 9/18/2015 21:48'!
hasXMLNSDeclarations
	^ xmlnsDeclarations notNil and: [xmlnsDeclarations notEmpty]! !

!XMLUnsafeStartTagWriter methodsFor: 'testing' stamp: 'monty 9/18/2015 21:26'!
includesAttribute: anAttribute
	^ self attributes includesKey: anAttribute! !

!XMLUnsafeStartTagWriter methodsFor: 'testing' stamp: 'monty 9/18/2015 21:27'!
includesXMLNS: aPrefix
	^ self xmlnsDeclarations includesKey: aPrefix! !

!XMLUnsafeStartTagWriter methodsFor: 'private' stamp: 'monty 9/18/2015 21:26'!
setAttributes: aDictionary
	attributes := aDictionary! !

!XMLUnsafeStartTagWriter methodsFor: 'private' stamp: 'monty 9/18/2015 21:26'!
setXMLNSDeclarations: aDictionary 
	xmlnsDeclarations := aDictionary! !

!XMLUnsafeStartTagWriter methodsFor: 'private' stamp: 'monty 9/18/2015 22:03'!
writeBody
	self writer
		writeName: self name;
		writeXMLNSDeclarations: xmlnsDeclarations;
		writeAttributes: attributes! !

!XMLUnsafeStartTagWriter methodsFor: 'private' stamp: 'monty 9/18/2015 22:03'!
writeEpilogue
	self writer
		nextPut: $>;
		formatAfterStartTag: self! !

!XMLUnsafeStartTagWriter methodsFor: 'private' stamp: 'monty 9/18/2015 22:03'!
writePrologue
	self writer
		formatBeforeStartTag: self;
		nextPut: $<! !

!XMLUnsafeStartTagWriter methodsFor: 'accessing' stamp: 'monty 9/18/2015 21:27'!
xmlns
	^ self xmlnsDeclarations at: ''! !

!XMLUnsafeStartTagWriter methodsFor: 'accessing' stamp: 'monty 9/18/2015 21:27'!
xmlns: aURI
	self xmlnsDeclarations at: '' put: aURI! !

!XMLUnsafeStartTagWriter methodsFor: 'accessing' stamp: 'monty 9/18/2015 21:26'!
xmlnsAt: aPrefix
	^ self xmlnsDeclarations at: aPrefix ifAbsent: ['']! !

!XMLUnsafeStartTagWriter methodsFor: 'accessing' stamp: 'monty 9/18/2015 21:26'!
xmlnsAt: aPrefix ifAbsent: aBlock
	^ self xmlnsDeclarations at: aPrefix ifAbsent: aBlock! !

!XMLUnsafeStartTagWriter methodsFor: 'accessing' stamp: 'monty 9/18/2015 21:26'!
xmlnsAt: aPrefix ifAbsentPut: aBlock
	^ self xmlnsDeclarations at: aPrefix ifAbsentPut: aBlock! !

!XMLUnsafeStartTagWriter methodsFor: 'accessing' stamp: 'monty 9/18/2015 21:26'!
xmlnsAt: aPrefix put: aValue
	^ self xmlnsDeclarations at: aPrefix put: aValue! !

!XMLUnsafeStartTagWriter methodsFor: 'accessing' stamp: 'monty 9/18/2015 21:26'!
xmlnsDeclarations
	^ xmlnsDeclarations
		ifNil: [xmlnsDeclarations := OrderPreservingStringDictionary new: 5]! !

!XMLUnsafeStartTagWriter methodsFor: 'accessing' stamp: 'monty 9/18/2015 21:26'!
xmlnsDeclarations: aDictionaryOrArray
	"set xmlns decls using a dictionary or array of prefix/uri associations.
	empty prefix is default xmlns."
	xmlnsDeclarations := OrderPreservingStringDictionary newFrom: aDictionaryOrArray! !

!XMLUnsafeTagWriter methodsFor: 'testing' stamp: 'monty 9/18/2015 21:20'!
isTagWriter
	^ true! !

!XMLUnsafeTagWriter methodsFor: 'testing' stamp: 'monty 9/18/2015 23:22'!
isUnsafeTagWriter
	^ true! !

!XMLUnsafeTagWriter methodsFor: 'accessing' stamp: 'monty 9/18/2015 21:07'!
name
	^ name ifNil: [name := '']! !

!XMLUnsafeTagWriter methodsFor: 'accessing' stamp: 'monty 9/18/2015 21:07'!
name: aString
	name := aString! !

XMLMarkupWriter subclass: #XMLXMLDeclarationWriter
	instanceVariableNames: 'encoding version standalone'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Writer-Core'!

!XMLXMLDeclarationWriter commentStamp: 'monty 4/17/2016 07:17' prior: 0!
This class writes <?xml ...?> declarations, but not if canonical XML is enabled.!

!XMLXMLDeclarationWriter methodsFor: 'accessing' stamp: 'monty 6/15/2015 04:10'!
encoding
	^ encoding ifNil: [encoding := '']! !

!XMLXMLDeclarationWriter methodsFor: 'accessing' stamp: 'JAAyer 7/26/2010 14:13'!
encoding: anEncoding
	encoding := anEncoding! !

!XMLXMLDeclarationWriter methodsFor: 'accessing' stamp: 'monty 11/19/2013 05:43'!
standalone
	^ standalone ifNil: [standalone := false]! !

!XMLXMLDeclarationWriter methodsFor: 'accessing' stamp: 'monty 12/16/2014 19:27'!
standalone: aBooleanOrString
	aBooleanOrString isString
		ifTrue: [standalone := aBooleanOrString = 'yes']
		ifFalse: [standalone := aBooleanOrString].! !

!XMLXMLDeclarationWriter methodsFor: 'accessing' stamp: 'JAAyer 7/26/2010 14:11'!
version
	^ version ifNil: [version := '1.0']! !

!XMLXMLDeclarationWriter methodsFor: 'accessing' stamp: 'monty 6/15/2015 04:18'!
version: aVersionFloatOrString
	| versionString |

	versionString := (aVersionFloatOrString ifNil: ['1.0']) asString.
	version :=
		versionString "cutoff excess digits without first: for GS portability"
			copyFrom: 1
			to: (3 min: versionString size)! !

!XMLXMLDeclarationWriter methodsFor: 'private' stamp: 'monty 11/8/2016 10:48'!
writeBody
	self isCanonical
		ifTrue: [^ self].

	self writer
		space;
		writeName: 'version'
		unescapedValue: (self version ifEmpty: ['1.0']).

	self encoding
		ifNotEmpty: [:encodingName |
			self writer
				space;
				writeName: 'encoding'
				unescapedValue: encodingName].

	self standalone
		ifTrue: [
			self writer
				space;
				writeName: 'standalone' unescapedValue: 'yes'].! !

!XMLXMLDeclarationWriter methodsFor: 'private' stamp: 'monty 11/8/2016 10:49'!
writeEpilogue
	self isCanonical
		ifFalse: [
			self writer
				nextPutAll: '?>';
				formatAfterXMLDeclaration: self]! !

!XMLXMLDeclarationWriter methodsFor: 'private' stamp: 'monty 11/8/2016 10:49'!
writePrologue
	self isCanonical
		ifFalse: [
			self writer
				formatBeforeXMLDeclaration: self;
				nextPutAll: '<?xml']! !

Object subclass: #XMLWriter
	instanceVariableNames: 'stream streamStartPosition markupWriterStack formatter lineBreak contentEscapes contentStringEscapes isEscaping isCanonical outputsSelfClosingTags omitsRedundantXMLNSDeclarations hadActiveTagWriter'
	classVariableNames: 'CanonicalLineBreak ContentCanonicalEscapes ContentEscapes ContentNonEntityEscapes ContentStringCanonicalEscapes ContentStringEscapes ContentStringNonEntityEscapes DefaultLineBreak EntityValueEscapes PlatformSpecificLineBreak SpaceCharacter'
	poolDictionaries: ''
	category: 'XML-Writer-Core'!

!XMLWriter commentStamp: 'monty 9/6/2015 15:13' prior: 0!
XMLWriter generates well-formed XML documents using an API similar to Seaside's canvas and tag brushes.

Markup can be created with the messages under "writing." The ones that take no arguments return instances of XMLMarkupWriter subclasses that can be configured with messages like #name: or #attributeAt:put:. Sending them #write or #with: (if they can have embedded markup) causes the markup writer's configuration to be committed and its markup to be written.

For markup writers that can have embedded markup, the argument to #with: can be a string, a block, a collection (of strings and blocks), or nil. A block can optionally take an argument, the XMLWriter. Creating additional markup will cause any unwritten markup from a previous markup writer to be written, as will sending #write, #contents, #printOn:, #asString, or #close.

Here is an example (evaluate with cmd-p):
	| writer |
	writer := XMLWriter new.
	writer
		enablePrettyPrinting;
		xml;
		comment: 'an example'.
	writer tag
		name: 'example:user';
		xmlnsAt: 'example' put: 'urn://example';
		with: [
			writer
				tag: 'username' with: 'jsmith';
				tag: 'real-name' with: 'John Smith';
				tag: 'age' with: '25';
				tag: 'id' with: '101']

If the last message sent isn't a writing message like #with: or #tag:with:, you may need to send #write to force writing any unwritten markup:
	String streamContents: [:stream |
		(XMLWriter on: stream)
			tag: 'first';
			tag: 'second';
			write].

The second tag forces the first to be written, but the second wouldn't be written if #write wasn't sent.

The #writeWith: message evaluates a block and automatically sends #write after, like how #fileNamed:do: automatically sends #close after:
	String streamContents: [:stream |
		(XMLWriter  on: stream)
			writeWith: [:writer |
				writer
					tag: 'first'
					with: [writer tag: 'second'].
				writer tag: 'third']].!

!XMLWriter class methodsFor: 'accessing' stamp: 'monty 11/8/2016 08:03'!
canonicalLineBreak
	^ CanonicalLineBreak! !

!XMLWriter class methodsFor: 'accessing' stamp: 'monty 11/8/2016 08:03'!
canonicalLineBreak: aCharacterOrString
	CanonicalLineBreak := aCharacterOrString asString! !

!XMLWriter class methodsFor: 'private' stamp: 'monty 11/8/2016 06:14'!
contentCanonicalEscapes
	^ ContentCanonicalEscapes! !

!XMLWriter class methodsFor: 'private' stamp: 'monty 11/8/2016 06:14'!
contentEscapes
	^ ContentEscapes! !

!XMLWriter class methodsFor: 'private' stamp: 'monty 11/8/2016 06:15'!
contentNonEntityEscapes
	^ ContentNonEntityEscapes! !

!XMLWriter class methodsFor: 'private' stamp: 'monty 11/8/2016 06:14'!
contentStringCanonicalEscapes
	^ ContentStringCanonicalEscapes! !

!XMLWriter class methodsFor: 'private' stamp: 'monty 11/8/2016 06:14'!
contentStringEscapes
	^ ContentStringEscapes! !

!XMLWriter class methodsFor: 'private' stamp: 'monty 11/8/2016 06:15'!
contentStringNonEntityEscapes
	^ ContentStringNonEntityEscapes! !

!XMLWriter class methodsFor: 'accessing' stamp: 'monty 4/17/2016 05:42'!
defaultLineBreak
	^ DefaultLineBreak! !

!XMLWriter class methodsFor: 'accessing' stamp: 'monty 4/17/2016 05:42'!
defaultLineBreak: aCharacterOrString
	DefaultLineBreak := aCharacterOrString asString! !

!XMLWriter class methodsFor: 'class initialization' stamp: 'monty 11/8/2016 08:07'!
initialize
	"self initialize"

	self
		initializeDefaultLineBreak;
		initializeCanonicalLineBreak;
		initializePlatformSpecificLineBreak;
		initializeContentEscapes;
		initializeContentCanonicalEscapes;
		initializeContentNonEntityEscapes;
		initializeContentStringEscapes;
		initializeContentStringCanonicalEscapes;
		initializeContentStringNonEntityEscapes;
		initializeEntityValueEscapes;
		initializeSpaceCharacter.! !

XMLWriter initialize!

!XMLWriter class methodsFor: 'class initialization' stamp: 'monty 11/8/2016 08:04'!
initializeCanonicalLineBreak
	self canonicalLineBreak: Character lf! !

!XMLWriter class methodsFor: 'class initialization' stamp: 'monty 11/9/2016 12:02'!
initializeContentCanonicalEscapes
	ContentCanonicalEscapes :=
		ContentEscapes copy
			at: 16rD put: '&#xD;'; "CR"
			yourself! !

!XMLWriter class methodsFor: 'class initialization' stamp: 'monty 10/4/2016 21:39'!
initializeContentEscapes
	"use an array instead of a dictionary for speed, and
	assign after with #yourself to avoid possible race
	conditions when reinitializing the class"
	ContentEscapes :=
		(Array new: 255)
			at: $& asciiValue put: '&amp;';
			at: $< asciiValue put: '&lt;';
			at: $> asciiValue put: '&gt;'; "required to escape ]]> in PCDATA"
			at: Character cr asciiValue put: #LineBreak;
			at: Character lf asciiValue put: #LineBreak;
			yourself! !

!XMLWriter class methodsFor: 'class initialization' stamp: 'monty 10/4/2016 21:41'!
initializeContentNonEntityEscapes
	"Like ContentEscapes, but allows entity references.
	Use an array instead of a dictionary for speed, and assign
	after with #yourself to avoid possible race conditions
	when reinitializing the class."
	ContentNonEntityEscapes :=
		ContentEscapes copy
			at: $& asciiValue put: nil;
			yourself! !

!XMLWriter class methodsFor: 'class initialization' stamp: 'monty 11/9/2016 12:02'!
initializeContentStringCanonicalEscapes
	ContentStringCanonicalEscapes :=
		ContentStringEscapes copy
			at: 16r9 put: '&#x9;'; "tab"
			at: 16rA put: '&#xA;'; "lf"
			at: 16rD put: '&#xD;'; "cr"
			yourself! !

!XMLWriter class methodsFor: 'class initialization' stamp: 'monty 10/4/2016 21:40'!
initializeContentStringEscapes
	"use an array instead of a dictionary for speed, and
	assign after with #yourself to avoid possible race
	conditions when reinitializing the class"
	ContentStringEscapes :=
		(Array new: 255)
			at: $& asciiValue put: '&amp;';
			at: $< asciiValue put: '&lt;'; "< is not allowed in attr values, but > is"
			at: $" asciiValue put: '&quot;';
			at: Character cr asciiValue put: #LineBreak;
			at: Character lf asciiValue put: #LineBreak;
			yourself! !

!XMLWriter class methodsFor: 'class initialization' stamp: 'monty 10/4/2016 21:41'!
initializeContentStringNonEntityEscapes
	"Like ContentStringEscapes, but allows entity references.
	Use an array instead of a dictionary for speed, and assign
	after with #yourself to avoid possible race conditions
	when reinitializing the class."
	ContentStringNonEntityEscapes :=
		ContentStringEscapes copy
			at: $& asciiValue put: nil;
			yourself! !

!XMLWriter class methodsFor: 'class initialization' stamp: 'monty 4/17/2016 05:43'!
initializeDefaultLineBreak
	self defaultLineBreak: Character cr! !

!XMLWriter class methodsFor: 'class initialization' stamp: 'monty 10/4/2016 21:40'!
initializeEntityValueEscapes
	"use an array instead of a dictionary for speed, and
	assign after with #yourself to avoid possible race
	conditions when reinitializing the class"
	EntityValueEscapes :=
		(Array new: 255)
			at: $" asciiValue put: '&#34;';
			at: $% asciiValue put: '&#37;';
			at: Character cr asciiValue put: #LineBreak;
			at: Character lf asciiValue put: #LineBreak;
			yourself! !

!XMLWriter class methodsFor: 'class initialization' stamp: 'monty 7/5/2016 07:36'!
initializePlatformSpecificLineBreak
	"'Smalltalk os platformName' is used for compatibility with GS and Squeak
	which don't have the OSPlatform classes, and also instead of the shorter
	'Smalltalk platformName' which doesn't work in all Pharos"
	(Smalltalk os platformName asLowercase beginsWith: 'win')
		ifTrue: [self platformSpecificLineBreak: String crlf]
		ifFalse: [self platformSpecificLineBreak: String lf].! !

!XMLWriter class methodsFor: 'class initialization' stamp: 'monty 5/18/2015 07:07'!
initializeSpaceCharacter
	SpaceCharacter := Character space! !

!XMLWriter class methodsFor: 'instance creation' stamp: 'JAAyer 7/27/2010 13:01'!
new
	^ self on: (String new: 512) writeStream! !

!XMLWriter class methodsFor: 'instance creation' stamp: 'monty 5/6/2015 01:29'!
on: aStream
	^ self basicNew initialize on: aStream! !

!XMLWriter class methodsFor: 'instance creation' stamp: 'monty 5/6/2015 01:26'!
on: aStream formatter: aFormatter
	^ (self on: aStream) formatter: aFormatter! !

!XMLWriter class methodsFor: 'accessing' stamp: 'monty 7/3/2016 02:10'!
platformSpecificLineBreak
	^ PlatformSpecificLineBreak! !

!XMLWriter class methodsFor: 'accessing' stamp: 'monty 7/3/2016 02:10'!
platformSpecificLineBreak: aCharacterOrString
	PlatformSpecificLineBreak := aCharacterOrString asString! !

!XMLWriter class methodsFor: 'writing' stamp: 'monty 10/26/2014 10:17'!
writeWith: aBlock
	^ self new
		writeWith: aBlock;
		contents! !

!XMLWriter methodsFor: 'writing' stamp: 'monty 10/12/2014 06:57'!
% aParameterEntityReference
	self paramEntityRef: aParameterEntityReference! !

!XMLWriter methodsFor: 'writing' stamp: 'monty 10/12/2014 06:57'!
& anEntityReference
	self entityRef: anEntityReference! !

!XMLWriter methodsFor: 'accessing' stamp: 'monty 11/27/2016 22:39'!
activeDoctypeDeclarationWriter
	self markupWritersDo: [:each |
		each isDoctypeDeclarationWriter
			ifTrue: [^ each]].
	^ nil.! !

!XMLWriter methodsFor: 'accessing' stamp: 'monty 4/17/2016 05:56'!
activeMarkupWriter
	| markupWriterStackSize |

	"optimized to avoid at:ifAbsent:/ifNotEmpty: closure"
	^ (markupWriterStackSize := markupWriterStack size) > 0
		ifTrue: [markupWriterStack at: markupWriterStackSize]
		ifFalse: [nil]! !

!XMLWriter methodsFor: 'private' stamp: 'monty 11/13/2016 21:48'!
activeMarkupWriterWhileChangingDo: aBlock
	"evaluate aBlock for the active (top) markup writer until there isn't
	one or it's the same (still active) writer after evaluating aBlock"

	| markupWriterStackSize activeMarkupWriter previousActiveMarkupWriter |

	"inlined from activeMarkupWriter for speed"
	[(markupWriterStackSize := markupWriterStack size) = 0
		or: [
			(activeMarkupWriter := markupWriterStack at: markupWriterStackSize) ==
				previousActiveMarkupWriter]]
		whileFalse: [
			aBlock value: activeMarkupWriter.
			previousActiveMarkupWriter := activeMarkupWriter]! !

!XMLWriter methodsFor: 'accessing' stamp: 'monty 11/27/2016 22:39'!
activeTagWriter
	self markupWritersDo: [:each |
		each isTagWriter
			ifTrue: [^ each]].
	^ nil.! !

!XMLWriter methodsFor: 'enumerating' stamp: 'monty 11/13/2016 21:48'!
ancestorMarkupWritersDo: aBlock
	self markupWriterStack size - 1 to: 1 by: -1 do: [:i |
		aBlock value: (self markupWriterStack at: i)]! !

!XMLWriter methodsFor: 'converting' stamp: 'monty 4/20/2016 07:57'!
asString
	"returns the full, untruncated output, unlike #printString, which may
	truncate"

	^ self contents! !

!XMLWriter methodsFor: 'testing' stamp: 'monty 7/4/2016 06:30'!
atBeginning
	"On GS streams can be 0-based or 1-based, so saving the initial position
	to test the current position works for both types and also handles streams
	that were already written to when the XMLWriter was created"

	^ stream position = streamStartPosition! !

!XMLWriter methodsFor: 'writing' stamp: 'monty 1/13/2014 23:52'!
attributeDeclaration
	^ self pushNewMarkupWriter: XMLAttributeDeclarationWriter! !

!XMLWriter methodsFor: 'writing' stamp: 'monty 12/1/2014 04:05'!
cdata
	^ self pushNewMarkupWriter: XMLCDataWriter! !

!XMLWriter methodsFor: 'writing' stamp: 'monty 12/1/2014 04:05'!
cdata: aString
	self cdata
		string: aString;
		write! !

!XMLWriter methodsFor: 'closing' stamp: 'monty 5/6/2015 16:38'!
close
	self write.
	stream close.! !

!XMLWriter methodsFor: 'writing' stamp: 'monty 12/1/2014 04:04'!
comment
	^ self pushNewMarkupWriter: XMLCommentWriter! !

!XMLWriter methodsFor: 'writing' stamp: 'monty 12/1/2014 04:04'!
comment: aString
	self comment
		string: aString;
		write! !

!XMLWriter methodsFor: 'private' stamp: 'monty 11/7/2016 03:35'!
contentEscapes
	^ contentEscapes! !

!XMLWriter methodsFor: 'private' stamp: 'monty 11/7/2016 03:35'!
contentEscapes: aContentEscapesTable
	contentEscapes := aContentEscapesTable! !

!XMLWriter methodsFor: 'private' stamp: 'monty 11/7/2016 03:35'!
contentStringEscapes
	^ contentStringEscapes! !

!XMLWriter methodsFor: 'private' stamp: 'monty 11/7/2016 03:35'!
contentStringEscapes: aContentStringEscapesTable
	contentStringEscapes := aContentStringEscapesTable! !

!XMLWriter methodsFor: 'accessing' stamp: 'monty 4/20/2016 07:57'!
contents
	"returns the full, untruncated output, unlike #printString, which may
	truncate"

	self write.
	^ stream contents.! !

!XMLWriter methodsFor: 'writing' stamp: 'monty 11/9/2016 12:34'!
doctype
	"an abbreviation"

	^ self doctypeDeclaration! !

!XMLWriter methodsFor: 'writing' stamp: 'monty 11/9/2016 12:34'!
doctypeDeclaration
	^ self pushNewMarkupWriter: XMLDoctypeDeclarationWriter! !

!XMLWriter methodsFor: 'writing' stamp: 'monty 1/13/2014 23:52'!
elementDeclaration
	^ self pushNewMarkupWriter: XMLElementDeclarationWriter! !

!XMLWriter methodsFor: 'writing' stamp: 'monty 7/4/2016 05:53'!
enablePlatformSpecificLineBreak
	"use the correct line break (LF or CRLF) for this platform when writing"

	self lineBreak: self class platformSpecificLineBreak! !

!XMLWriter methodsFor: 'writing' stamp: 'monty 7/4/2016 05:53'!
enablePrettyPrinting
	"enables 'pretty printing' whitespace formatting to make nicer output"

	self formatter: (XMLPrettyPrintingWriterFormatter writer: self)! !

!XMLWriter methodsFor: 'writing' stamp: 'monty 12/21/2015 21:49'!
endTag
	"Returns a new XMLUnsafeEndTagWriter for writing an end tag to close an element
	started by a #startTag* message.

	Use the #tag* messages instead if possible."

	^ self pushNewMarkupWriter: XMLUnsafeEndTagWriter! !

!XMLWriter methodsFor: 'writing' stamp: 'monty 12/21/2015 21:49'!
endTag: aName
	"Returns a new XMLUnsafeEndTagWriter for writing an end tag with the specified
	name to close an element started by a #startTag* message.

	Use the #tag* messages instead if possible."

	^ self endTag name: aName! !

!XMLWriter methodsFor: 'writing' stamp: 'monty 1/13/2014 23:52'!
entityDeclaration
	^ self pushNewMarkupWriter: XMLEntityDeclarationWriter! !

!XMLWriter methodsFor: 'writing' stamp: 'monty 5/8/2015 04:50'!
entityRef
	^ self pushNewMarkupWriter: XMLEntityReferenceWriter! !

!XMLWriter methodsFor: 'writing' stamp: 'monty 7/3/2016 23:59'!
entityRef: aName
	"output a general entity reference like &name;"

	self entityRef
		name: aName;
		write! !

!XMLWriter methodsFor: 'testing' stamp: 'monty 11/8/2016 06:29'!
escapesContentEntityReferences
	^ self isEscaping
		and: [(contentEscapes at: $& asciiValue) notNil
			and: [(contentStringEscapes at: $& asciiValue) notNil]]! !

!XMLWriter methodsFor: 'accessing' stamp: 'monty 11/8/2016 06:25'!
escapesContentEntityReferences: aBoolean
	self escapesContentEntityReferences = aBoolean
		ifTrue: [^ self].
	aBoolean
		ifTrue: [
			self
				isEscaping: true;
				contentEscapes: ContentEscapes;
				contentStringEscapes: ContentStringEscapes]
		ifFalse: [
			self
				contentEscapes: ContentNonEntityEscapes;
				contentStringEscapes: ContentStringNonEntityEscapes]! !

!XMLWriter methodsFor: 'formatting' stamp: 'monty 9/20/2015 02:54'!
formatAfterCData: aCDataWriter
	formatter
		ifNotNil: [formatter formatAfterCData: aCDataWriter]! !

!XMLWriter methodsFor: 'formatting' stamp: 'monty 9/20/2015 21:33'!
formatAfterCharacterEscape: aCharacterEscape
	formatter
		ifNotNil: [formatter formatAfterCharacterEscape: aCharacterEscape]! !

!XMLWriter methodsFor: 'formatting' stamp: 'monty 5/8/2015 04:10'!
formatAfterComment: aCommentWriter
	formatter
		ifNotNil: [formatter formatAfterComment: aCommentWriter]! !

!XMLWriter methodsFor: 'formatting' stamp: 'monty 5/8/2015 04:14'!
formatAfterDeclaration: aDeclarationWriter
	formatter
		ifNotNil: [formatter formatAfterDeclaration: aDeclarationWriter]! !

!XMLWriter methodsFor: 'formatting' stamp: 'monty 12/1/2014 23:45'!
formatAfterDoctypeDeclaration: aDoctypeDeclarationWriter
	formatter
		ifNotNil: [formatter formatAfterDoctypeDeclaration: aDoctypeDeclarationWriter]! !

!XMLWriter methodsFor: 'formatting' stamp: 'monty 5/8/2015 04:08'!
formatAfterEndTag: aTagWriter
	formatter
		ifNotNil: [formatter formatAfterEndTag: aTagWriter]! !

!XMLWriter methodsFor: 'formatting' stamp: 'monty 5/8/2015 04:15'!
formatAfterEntityReference: anEntityReferenceWriter
	formatter
		ifNotNil: [formatter formatAfterEntityReference: anEntityReferenceWriter]! !

!XMLWriter methodsFor: 'formatting' stamp: 'monty 12/1/2014 23:42'!
formatAfterInternalSubset: aDoctypeDeclarationWriter
	formatter
		ifNotNil: [formatter formatAfterInternalSubset: aDoctypeDeclarationWriter]! !

!XMLWriter methodsFor: 'formatting' stamp: 'monty 5/8/2015 04:11'!
formatAfterPI: aPIWriter
	formatter
		ifNotNil: [formatter formatAfterPI: aPIWriter]! !

!XMLWriter methodsFor: 'formatting' stamp: 'monty 9/18/2015 18:52'!
formatAfterQuotedStringValue: aString
	formatter
		ifNotNil: [formatter formatAfterQuotedStringValue: aString]! !

!XMLWriter methodsFor: 'formatting' stamp: 'monty 12/1/2014 23:34'!
formatAfterStartTag: aTagWriter
	formatter
		ifNotNil: [formatter formatAfterStartTag: aTagWriter]! !

!XMLWriter methodsFor: 'formatting' stamp: 'monty 5/8/2015 04:10'!
formatAfterXMLDeclaration: anXMLDeclarationWriter
	formatter
		ifNotNil: [formatter formatAfterXMLDeclaration: anXMLDeclarationWriter]! !

!XMLWriter methodsFor: 'formatting' stamp: 'monty 9/20/2015 02:54'!
formatBeforeCData: aCDataWriter
	formatter
		ifNotNil: [formatter formatBeforeCData: aCDataWriter]! !

!XMLWriter methodsFor: 'formatting' stamp: 'monty 9/20/2015 21:33'!
formatBeforeCharacterEscape: aCharacterEscape
	formatter
		ifNotNil: [formatter formatBeforeCharacterEscape: aCharacterEscape]! !

!XMLWriter methodsFor: 'formatting' stamp: 'monty 12/1/2014 23:39'!
formatBeforeComment: aCommentWriter
	formatter
		ifNotNil: [formatter formatBeforeComment: aCommentWriter]! !

!XMLWriter methodsFor: 'formatting' stamp: 'monty 12/1/2014 23:39'!
formatBeforeDeclaration: aDeclarationWriter
	formatter
		ifNotNil: [formatter formatBeforeDeclaration: aDeclarationWriter]! !

!XMLWriter methodsFor: 'formatting' stamp: 'monty 12/1/2014 23:44'!
formatBeforeDoctypeDeclaration: aDoctypeDeclarationWriter
	formatter
		ifNotNil: [formatter formatBeforeDoctypeDeclaration: aDoctypeDeclarationWriter]! !

!XMLWriter methodsFor: 'formatting' stamp: 'monty 12/1/2014 23:34'!
formatBeforeEndTag: aTagWriter
	formatter
		ifNotNil: [formatter formatBeforeEndTag: aTagWriter]! !

!XMLWriter methodsFor: 'formatting' stamp: 'monty 12/2/2014 05:41'!
formatBeforeEntityReference: anEntityReferenceWriter
	formatter
		ifNotNil: [formatter formatBeforeEntityReference: anEntityReferenceWriter]! !

!XMLWriter methodsFor: 'formatting' stamp: 'monty 12/1/2014 23:46'!
formatBeforeInternalSubset: aDoctypeDeclarationWriter
	formatter
		ifNotNil: [formatter formatBeforeInternalSubset: aDoctypeDeclarationWriter]! !

!XMLWriter methodsFor: 'formatting' stamp: 'monty 5/8/2015 04:12'!
formatBeforePI: aPIWriter
	formatter
		ifNotNil: [formatter formatBeforePI: aPIWriter]! !

!XMLWriter methodsFor: 'formatting' stamp: 'monty 9/18/2015 18:51'!
formatBeforeQuotedStringValue: aString
	formatter
		ifNotNil: [formatter formatBeforeQuotedStringValue: aString]! !

!XMLWriter methodsFor: 'formatting' stamp: 'monty 12/1/2014 23:34'!
formatBeforeStartTag: aTagWriter
	formatter
		ifNotNil: [formatter formatBeforeStartTag: aTagWriter]! !

!XMLWriter methodsFor: 'formatting' stamp: 'monty 3/24/2015 19:34'!
formatBeforeXMLDeclaration: anXMLDeclarationWriter
	formatter
		ifNotNil: [formatter formatBeforeXMLDeclaration: anXMLDeclarationWriter]! !

!XMLWriter methodsFor: 'accessing' stamp: 'monty 11/30/2014 04:03'!
formatter
	^ formatter! !

!XMLWriter methodsFor: 'accessing' stamp: 'monty 11/8/2016 06:11'!
formatter: aFormatter
	(formatter := aFormatter)
		ifNotNil: [formatter writer: self]! !

!XMLWriter methodsFor: 'testing' stamp: 'monty 11/8/2016 07:07'!
hadActiveTagWriter
	^ hadActiveTagWriter! !

!XMLWriter methodsFor: 'testing' stamp: 'monty 11/8/2016 07:14'!
hasActiveTagWriter
	^ self activeTagWriter notNil! !

!XMLWriter methodsFor: 'testing' stamp: 'monty 11/22/2014 09:39'!
isCanonical
	^ isCanonical! !

!XMLWriter methodsFor: 'accessing' stamp: 'monty 11/8/2016 08:05'!
isCanonical: aBoolean
	"Should the output be canonical XML (http://www.w3.org/TR/xml-c14n)?
	(Not default)"

	self isCanonical = aBoolean
		ifTrue: [^ self].
	(isCanonical := aBoolean)
		ifTrue: [
			self
				formatter: XMLCanonicalWriterFormatter new;
				lineBreak: self class canonicalLineBreak;
				contentEscapes: ContentCanonicalEscapes;
				contentStringEscapes: ContentStringCanonicalEscapes;
				isEscaping: true;
				omitsRedundantXMLNSDeclarations: true]
		ifFalse: [
			self
				formatter: nil;
				lineBreak: self class defaultLineBreak;
				contentEscapes: ContentEscapes;
				contentStringEscapes: ContentStringEscapes]! !

!XMLWriter methodsFor: 'testing' stamp: 'monty 11/22/2014 09:41'!
isEscaping
	^ isEscaping! !

!XMLWriter methodsFor: 'accessing' stamp: 'monty 7/4/2016 05:45'!
isEscaping: aBoolean
	"can be used to disable escaping of special characters and the
	replacement of CRs/LFs in attribute/entity values and strings
	with whatever #lineBreak is set to"

	isEscaping := aBoolean! !

!XMLWriter methodsFor: 'private' stamp: 'monty 11/20/2016 13:57'!
isRedundantXMLNSPrefix: aPrefix uri: aURI
	self ancestorMarkupWritersDo: [:each |
		(each isTagWriter
			and: [each hasXMLNSDeclarations])
			ifTrue: [
				each xmlnsDeclarations
					at: aPrefix
					ifPresent: [:uri | ^ uri = aURI]]].
	^ aPrefix size = 0
		and: [aURI size = 0].! !

!XMLWriter methodsFor: 'accessing' stamp: 'monty 12/1/2014 06:32'!
lineBreak
	^ lineBreak! !

!XMLWriter methodsFor: 'accessing' stamp: 'monty 4/20/2016 07:58'!
lineBreak: aCharacterOrString
	"linebreak chars (CR/LF) in attribute/entity values and in args to
	the #string: and #with: messages are replaced with whatever char/string
	this is set to"

	lineBreak := aCharacterOrString asString! !

!XMLWriter methodsFor: 'private' stamp: 'monty 10/27/2014 08:19'!
markupWriterStack
	^ markupWriterStack! !

!XMLWriter methodsFor: 'enumerating' stamp: 'monty 11/27/2016 22:39'!
markupWritersDo: aBlock
	self markupWriterStack reverseDo: aBlock! !

!XMLWriter methodsFor: 'private' stamp: 'monty 11/13/2016 21:48'!
newStringMarkup
	self activeMarkupWriterWhileChangingDo: [:each | each newStringMarkupBeingWritten]! !

!XMLWriter methodsFor: 'private' stamp: 'monty 12/5/2014 04:07'!
nextPut: aCharacter
	^ stream nextPut: aCharacter! !

!XMLWriter methodsFor: 'private' stamp: 'monty 12/5/2014 04:07'!
nextPutAll: aString
	^ stream nextPutAll: aString! !

!XMLWriter methodsFor: 'writing' stamp: 'monty 1/13/2014 23:52'!
notationDeclaration
	^ self pushNewMarkupWriter: XMLNotationDeclarationWriter! !

!XMLWriter methodsFor: 'testing' stamp: 'monty 3/24/2015 19:27'!
omitsRedundantXMLNSDeclarations
	^ omitsRedundantXMLNSDeclarations! !

!XMLWriter methodsFor: 'accessing' stamp: 'monty 3/24/2015 19:27'!
omitsRedundantXMLNSDeclarations: aBoolean
	omitsRedundantXMLNSDeclarations := aBoolean! !

!XMLWriter methodsFor: 'initialization' stamp: 'monty 11/8/2016 07:07'!
on: aStream
	stream := aStream.
	streamStartPosition := aStream position.
	markupWriterStack := OrderedCollection new.
	lineBreak := DefaultLineBreak.
	contentEscapes := ContentEscapes.
	contentStringEscapes := ContentStringEscapes.
	isEscaping := true.
	isCanonical := false.
	outputsSelfClosingTags := true.
	omitsRedundantXMLNSDeclarations := true.
	hadActiveTagWriter := false.! !

!XMLWriter methodsFor: 'testing' stamp: 'monty 11/22/2014 09:39'!
outputsSelfClosingTags
	^ outputsSelfClosingTags! !

!XMLWriter methodsFor: 'accessing' stamp: 'monty 4/20/2016 07:52'!
outputsSelfClosingTags: aBoolean
	"can be used to disable outputting empty elements as '<empty/>'
	instead of '<empty></empty>'"

	outputsSelfClosingTags := aBoolean! !

!XMLWriter methodsFor: 'writing' stamp: 'monty 5/8/2015 05:05'!
paramEntityRef
	^ (self pushNewMarkupWriter: XMLEntityReferenceWriter) isParameter: true! !

!XMLWriter methodsFor: 'writing' stamp: 'monty 7/3/2016 23:59'!
paramEntityRef: aName
	"output a parameter entity reference like %name;"

	self paramEntityRef
		name: aName;
		write! !

!XMLWriter methodsFor: 'accessing' stamp: 'monty 9/18/2015 21:58'!
parentMarkupWriter
	^ self activeMarkupWriter
		ifNotNil: [:activeMarkupWriter | self parentMarkupWriterOf: activeMarkupWriter]! !

!XMLWriter methodsFor: 'accessing' stamp: 'monty 12/2/2014 23:30'!
parentMarkupWriterOf: aMarkupWriter
	self markupWriterStack size to: 2 by: -1 do: [:i |
		(self markupWriterStack at: i) == aMarkupWriter
			ifTrue: [^ self markupWriterStack at: i - 1]].
	^ nil.! !

!XMLWriter methodsFor: 'writing' stamp: 'monty 4/16/2016 13:43'!
pcdata: aString
	self deprecated: 'use #string: instead'.
	self string: aString.! !

!XMLWriter methodsFor: 'writing' stamp: 'monty 4/20/2016 07:59'!
pi
	"returns a new processing instruction writer for writing"

	^ self pushNewMarkupWriter: XMLPIWriter! !

!XMLWriter methodsFor: 'private' stamp: 'monty 12/13/2014 10:53'!
popActiveMarkupWriter
	^ self markupWriterStack removeLast! !

!XMLWriter methodsFor: 'printing' stamp: 'monty 12/5/2014 03:51'!
printOn: aStream
	aStream nextPutAll: self contents! !

!XMLWriter methodsFor: 'private' stamp: 'monty 11/13/2016 21:48'!
pushNewMarkupWriter: aMarkupWriterClass
	| newMarkupWriter |

	self activeMarkupWriterWhileChangingDo: [:each |
		each newMarkupWriterBeingPushed].
	newMarkupWriter :=
		self markupWriterStack addLast:
			(aMarkupWriterClass writer: self).
	self hadActiveTagWriter
		ifFalse: [hadActiveTagWriter := newMarkupWriter isTagWriter].
	^ newMarkupWriter.! !

!XMLWriter methodsFor: 'writing' stamp: 'monty 4/20/2016 07:59'!
raw: aString
	"writes raw string content without doing any escaping of special
	characters, unlike #string:"

	aString size > 0
		ifTrue: [
			self
				newStringMarkup;
				nextPutAll: aString]! !

!XMLWriter methodsFor: 'private' stamp: 'monty 11/13/2016 17:19'!
resolveAttributePrefix: aPrefix
	^ self
		resolveAttributePrefix: aPrefix
		ifUnresolvable: [''].! !

!XMLWriter methodsFor: 'private' stamp: 'monty 11/27/2016 22:39'!
resolveAttributePrefix: aPrefix ifUnresolvable: aBlock
	aPrefix size > 0
		ifTrue: [
			aPrefix = 'xml'
				ifTrue: [^ 'http://www.w3.org/XML/1998/namespace'].
			self markupWritersDo: [:each |
				each isTagWriter
					and: [each hasXMLNSDeclarations
						and: [
							each xmlnsDeclarations
								at: aPrefix
								ifPresent: [:uri | ^ uri]]]]].
	^ aBlock value.! !

!XMLWriter methodsFor: 'private' stamp: 'monty 5/25/2015 07:11'!
space
	"redefined for speed and portability"
	stream nextPut: SpaceCharacter! !

!XMLWriter methodsFor: 'writing' stamp: 'monty 12/21/2015 21:54'!
startTag
	"Returns a new XMLUnsafeStartTagWriter for writing a start tag that does
	not support embedding or automatic end tag generation.

	Use the #tag* messages instead if possible."

	^ self pushNewMarkupWriter: XMLUnsafeStartTagWriter! !

!XMLWriter methodsFor: 'writing' stamp: 'monty 12/21/2015 21:54'!
startTag: aName
	"Returns a new XMLUnsafeStartTagWriter for writing a start tag with the
	specified name that does not support embedding or automatic end tag generation.

	Use the #tag* messages instead if possible."

	^ self startTag name: aName! !

!XMLWriter methodsFor: 'writing' stamp: 'monty 12/21/2015 21:54'!
startTag: aName attributes: aDictionaryOrArray
	"Returns a new XMLUnsafeStartTagWriter for writing a start tag with the
	specified name and attributes that does not support embedding or automatic
	end tag generation.
	
	Use the #tag* messages instead if possible."

	^ (self startTag: aName)
		attributes: aDictionaryOrArray! !

!XMLWriter methodsFor: 'private' stamp: 'monty 1/13/2014 23:14'!
stream
	^ stream! !

!XMLWriter methodsFor: 'private' stamp: 'monty 5/6/2015 16:05'!
stream: aStream
	self on: aStream! !

!XMLWriter methodsFor: 'writing' stamp: 'monty 4/20/2016 07:52'!
string: aString
	"writes string content while '&'-escaping any special characters"

	aString size > 0
		ifTrue: [
			self
				newStringMarkup;
				writeEscapedContent: aString]! !

!XMLWriter methodsFor: 'writing' stamp: 'monty 12/21/2015 21:46'!
tag
	"returns a new XMLTagWriter for writing an element that supports embedding
	markup using #with: and automatic end tag generation"

	^ self pushNewMarkupWriter: XMLTagWriter! !

!XMLWriter methodsFor: 'writing' stamp: 'monty 12/21/2015 21:46'!
tag: aName
	"returns a new XMLTagWriter for writing an element with the specified name
	that supports embedding markup using #with: and automatic end tag generation"

	^ self tag name: aName! !

!XMLWriter methodsFor: 'writing' stamp: 'monty 12/21/2015 21:46'!
tag: aName attributes: aDictionaryOrArray
	"returns a new XMLTagWriter for writing an element with the specified name and
	attributes that supports embedding markup using #with: and automatic end tag
	generation"

	^ (self tag: aName)
		attributes: aDictionaryOrArray! !

!XMLWriter methodsFor: 'writing' stamp: 'monty 12/21/2015 14:43'!
tag: aName attributes: aDictionaryOrArray with: aStringOrCollectionOrBlock
	"writes an element with the specified name, attributes, and embedded markup"

	^ (self tag: aName)
		attributes: aDictionaryOrArray;
		with: aStringOrCollectionOrBlock! !

!XMLWriter methodsFor: 'writing' stamp: 'monty 12/21/2015 14:43'!
tag: aName with: aStringOrCollectionOrBlock
	"writes an element with the specified name and embedded markup"

	^ (self tag: aName) with: aStringOrCollectionOrBlock! !

!XMLWriter methodsFor: 'writing' stamp: 'monty 11/13/2016 21:48'!
write
	"writes any unwritten markup, including any markup writer that hasn't
	been written yet"

	self activeMarkupWriterWhileChangingDo: [:each |
		each hasStartedWriting
			ifFalse: [each write]]! !

!XMLWriter methodsFor: 'private' stamp: 'monty 9/20/2015 21:36'!
write: aString escapedWith: anEscapeTable
	self isEscaping
		ifTrue: [| tableSize |
			tableSize := anEscapeTable size.
			"to:do: for performance"
			1 to: aString size do: [:i | | nextChar asciiValue |
				((asciiValue := (nextChar := aString at: i) asciiValue) > 0
					and: [asciiValue <= tableSize
						and: [(anEscapeTable at: asciiValue) notNil]])
					ifTrue: [| replacement |
						(replacement := anEscapeTable at: asciiValue) == #LineBreak
							ifTrue: [stream nextPutAll: self lineBreak]
							ifFalse: [
								self formatBeforeCharacterEscape: replacement.
								stream nextPutAll: replacement.
								self formatAfterCharacterEscape: replacement]]
					ifFalse: [stream nextPut: nextChar]]]
		ifFalse: [stream nextPutAll: aString]! !

!XMLWriter methodsFor: 'private' stamp: 'monty 9/21/2015 01:07'!
writeAttributeName: aName value: aValue 
	self
		space;
		writeName: aName escapedContentValue: aValue! !

!XMLWriter methodsFor: 'private' stamp: 'monty 11/13/2016 15:34'!
writeAttributes: anAttributeCollection
	anAttributeCollection
		ifNotNil: [
			self isCanonical
				ifTrue: [anAttributeCollection writeCanonicalAttributesOn: self]
				ifFalse: [anAttributeCollection writeAttributesOn: self]]! !

!XMLWriter methodsFor: 'private' stamp: 'monty 1/5/2015 07:40'!
writeEscapedContent: aString
	self
		write: aString
		escapedWith: contentEscapes! !

!XMLWriter methodsFor: 'private' stamp: 'monty 9/18/2015 18:51'!
writeEscapedContentString: aString
	self
		nextPut: $";
		formatBeforeQuotedStringValue: aString;
		write: aString escapedWith: contentStringEscapes;
		formatAfterQuotedStringValue: aString;
		nextPut: $"! !

!XMLWriter methodsFor: 'private' stamp: 'monty 9/18/2015 19:13'!
writeEscapedEntityValueString: aString
	self
		nextPut: $";
		formatBeforeQuotedStringValue: aString;
		write: aString escapedWith: EntityValueEscapes;
		formatAfterQuotedStringValue: aString;
		nextPut: $"! !

!XMLWriter methodsFor: 'private' stamp: 'monty 12/13/2014 10:42'!
writeName: aString
	stream nextPutAll: aString! !

!XMLWriter methodsFor: 'private' stamp: 'monty 12/13/2014 10:41'!
writeName: aName escapedContentValue: aValue
	self
		writeName: aName;
		nextPut: $=;
		writeEscapedContentString: aValue! !

!XMLWriter methodsFor: 'private' stamp: 'monty 12/13/2014 10:42'!
writeName: aName unescapedValue: aValue
	self
		writeName: aName;
		nextPut: $=;
		writeUnescapedString: aValue! !

!XMLWriter methodsFor: 'private' stamp: 'monty 9/18/2015 18:53'!
writeUnescapedString: aString
	| quote |

	quote :=
		(aString includes: $")
			ifTrue: [$']
			ifFalse: [$"].
	self
		nextPut: quote;
		formatBeforeQuotedStringValue: aString.
	aString do: [:each |
		"delimiters are skipped because they can't be escaped here"
		each == quote
			ifFalse: [self nextPut: each]].
	self	
		formatAfterQuotedStringValue: aString;
		nextPut: quote.! !

!XMLWriter methodsFor: 'private' stamp: 'monty 11/13/2016 21:48'!
writeUnwrittenMarkupWritersEmbeddedIn: aMarkupWriter
	self activeMarkupWriterWhileChangingDo: [:each |
		each == aMarkupWriter
			ifTrue: [^ self].
		each hasStartedWriting
			ifFalse: [each write]]! !

!XMLWriter methodsFor: 'writing' stamp: 'monty 4/20/2016 07:54'!
writeWith: aBlock
	"evaluates aBlock with the receiver as an optional argument and
	automatically sends #write after"

	aBlock cull: self.
	self write.! !

!XMLWriter methodsFor: 'private' stamp: 'monty 11/27/2016 21:11'!
writeXMLNS: aPrefix uri: aURI
	(self omitsRedundantXMLNSDeclarations
		and: [
			self
				isRedundantXMLNSPrefix: aPrefix
				uri: aURI])
		ifFalse: [
			self
				space;
				writeName:
					(aPrefix isEmpty
						ifTrue: ['xmlns']
						ifFalse: ['xmlns:', aPrefix])
				escapedContentValue: aURI]! !

!XMLWriter methodsFor: 'private' stamp: 'monty 11/27/2016 21:11'!
writeXMLNSDeclarations: aDictionary
	aDictionary
		ifNil: [^ self].

	self isCanonical
		ifTrue: [
			aDictionary keys sort do: [:prefix |
				self
					writeXMLNS: prefix
					uri: (aDictionary at: prefix)]]
		ifFalse: [
			aDictionary keysDo: [:prefix |
				self
					writeXMLNS: prefix
					uri: (aDictionary at: prefix)]].! !

!XMLWriter methodsFor: 'writing' stamp: 'monty 4/20/2016 07:53'!
xml
	"an abbreviation"

	^ self xmlDeclaration! !

!XMLWriter methodsFor: 'writing' stamp: 'monty 12/20/2015 23:09'!
xmlDeclaration
	^ self pushNewMarkupWriter: XMLXMLDeclarationWriter! !

Object subclass: #XMLWriterFormatter
	instanceVariableNames: 'writer indentString currentIndentLevel'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Writer-Core'!

!XMLWriterFormatter commentStamp: 'monty 5/8/2015 05:00' prior: 0!
An abstract class for formatters used to decorate markup writers. Override the "formatting" messages.!

XMLWriterFormatter subclass: #XMLCanonicalWriterFormatter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Writer-Core'!

!XMLCanonicalWriterFormatter commentStamp: 'monty 11/8/2016 08:02' prior: 0!
A formatter for adding LF linebreaks around canonical markup.!

!XMLCanonicalWriterFormatter methodsFor: 'formatting' stamp: 'monty 11/8/2016 07:27'!
formatAfterComment: aCommentWriter
	self writer hadActiveTagWriter
		ifFalse: [self writeLineBreak]! !

!XMLCanonicalWriterFormatter methodsFor: 'formatting' stamp: 'monty 11/8/2016 07:26'!
formatAfterPI: aPIWriter
	self writer hadActiveTagWriter
		ifFalse: [self writeLineBreak]! !

!XMLCanonicalWriterFormatter methodsFor: 'formatting' stamp: 'monty 11/8/2016 07:26'!
formatBeforeComment: aCommentWriter
	(self writer hadActiveTagWriter
		and: [self writer hasActiveTagWriter not])
		ifTrue: [self writeLineBreak]! !

!XMLCanonicalWriterFormatter methodsFor: 'formatting' stamp: 'monty 11/8/2016 07:26'!
formatBeforePI: aPIWriter
	(self writer hadActiveTagWriter
		and: [self writer hasActiveTagWriter not])
		ifTrue: [self writeLineBreak]! !

XMLWriterFormatter subclass: #XMLPrettyPrintingWriterFormatter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Writer-Core'!

!XMLPrettyPrintingWriterFormatter commentStamp: 'monty 7/3/2016 23:29' prior: 0!
A formatter that inserts whitespace formatting to make XML output nicer and more readable.!

!XMLPrettyPrintingWriterFormatter methodsFor: 'formatting' stamp: 'monty 12/1/2014 23:45'!
formatAfterDoctypeDeclaration: aDoctypeDeclarationWriter
	self writeLineBreak! !

!XMLPrettyPrintingWriterFormatter methodsFor: 'formatting' stamp: 'monty 12/1/2014 23:43'!
formatAfterInternalSubset: aDoctypeDeclarationWriter
	aDoctypeDeclarationWriter hasEmbeddedMarkup
		ifTrue: [
			self
				outdent;
				writeLineBreak]! !

!XMLPrettyPrintingWriterFormatter methodsFor: 'formatting' stamp: 'monty 12/1/2014 23:33'!
formatAfterStartTag: aTagWriter
	self indent! !

!XMLPrettyPrintingWriterFormatter methodsFor: 'formatting' stamp: 'monty 12/1/2014 23:39'!
formatBeforeComment: aCommentWriter
	aCommentWriter isEmbeddedWithStringMarkup
		ifFalse: [self writeLineBreak]! !

!XMLPrettyPrintingWriterFormatter methodsFor: 'formatting' stamp: 'monty 12/1/2014 23:40'!
formatBeforeDeclaration: aDeclarationWriter
	self writeLineBreak! !

!XMLPrettyPrintingWriterFormatter methodsFor: 'formatting' stamp: 'monty 12/1/2014 23:43'!
formatBeforeDoctypeDeclaration: aDoctypeDeclarationWriter
	self writeLineBreak! !

!XMLPrettyPrintingWriterFormatter methodsFor: 'formatting' stamp: 'monty 12/1/2014 23:33'!
formatBeforeEndTag: aTagWriter
	aTagWriter hasEmbeddedMarkup
		ifTrue: [
			self outdent.
			aTagWriter hasEmbeddedStringMarkup
				ifFalse: [self writeLineBreak]]! !

!XMLPrettyPrintingWriterFormatter methodsFor: 'formatting' stamp: 'monty 12/2/2014 00:36'!
formatBeforeEntityReference: anEntityReferenceWriter
	anEntityReferenceWriter isParameter
		ifTrue: [self writeLineBreak]! !

!XMLPrettyPrintingWriterFormatter methodsFor: 'formatting' stamp: 'monty 12/1/2014 23:46'!
formatBeforeInternalSubset: aDoctypeDeclarationWriter
	self indent! !

!XMLPrettyPrintingWriterFormatter methodsFor: 'formatting' stamp: 'monty 5/8/2015 04:12'!
formatBeforePI: aPIWriter
	aPIWriter isEmbeddedWithStringMarkup
		ifFalse: [self writeLineBreak]! !

!XMLPrettyPrintingWriterFormatter methodsFor: 'formatting' stamp: 'monty 12/1/2014 23:50'!
formatBeforeStartTag: aTagWriter
	aTagWriter isEmbeddedWithStringMarkup
		ifFalse: [self writeLineBreak]! !

!XMLWriterFormatter class methodsFor: 'private' stamp: 'monty 5/25/2015 06:54'!
defaultIndentLevel
	^ 0! !

!XMLWriterFormatter class methodsFor: 'private' stamp: 'monty 5/25/2015 06:54'!
defaultIndentString
	^ '    '! !

!XMLWriterFormatter class methodsFor: 'instance creation' stamp: 'monty 9/20/2015 23:24'!
writer: aWriter
	^ self new writer: aWriter! !

!XMLWriterFormatter methodsFor: 'private' stamp: 'monty 11/30/2014 03:25'!
currentIndentLevel
	^ currentIndentLevel! !

!XMLWriterFormatter methodsFor: 'formatting' stamp: 'monty 9/20/2015 02:55'!
formatAfterCData: aCDataWriter! !

!XMLWriterFormatter methodsFor: 'formatting' stamp: 'monty 9/20/2015 21:28'!
formatAfterCharacterEscape: aCharacterEscape! !

!XMLWriterFormatter methodsFor: 'formatting' stamp: 'monty 5/8/2015 04:09'!
formatAfterComment: aCommentWriter! !

!XMLWriterFormatter methodsFor: 'formatting' stamp: 'monty 5/8/2015 04:14'!
formatAfterDeclaration: aDeclarationWriter! !

!XMLWriterFormatter methodsFor: 'formatting' stamp: 'monty 12/1/2014 23:45'!
formatAfterDoctypeDeclaration: aDoctypeDeclarationWriter! !

!XMLWriterFormatter methodsFor: 'formatting' stamp: 'monty 5/8/2015 04:06'!
formatAfterEndTag: aTagWriter! !

!XMLWriterFormatter methodsFor: 'formatting' stamp: 'monty 5/8/2015 04:16'!
formatAfterEntityReference: anEntityReferenceWriter! !

!XMLWriterFormatter methodsFor: 'formatting' stamp: 'monty 12/1/2014 23:42'!
formatAfterInternalSubset: aDoctypeDeclarationWriter! !

!XMLWriterFormatter methodsFor: 'formatting' stamp: 'monty 5/8/2015 04:12'!
formatAfterPI: aPIWriter! !

!XMLWriterFormatter methodsFor: 'formatting' stamp: 'monty 9/18/2015 18:53'!
formatAfterQuotedStringValue: aString! !

!XMLWriterFormatter methodsFor: 'formatting' stamp: 'monty 12/1/2014 23:37'!
formatAfterStartTag: aTagWriter! !

!XMLWriterFormatter methodsFor: 'formatting' stamp: 'monty 5/8/2015 04:11'!
formatAfterXMLDeclaration: anXMLDeclarationWriter! !

!XMLWriterFormatter methodsFor: 'formatting' stamp: 'monty 9/20/2015 02:55'!
formatBeforeCData: aCDataWriter! !

!XMLWriterFormatter methodsFor: 'formatting' stamp: 'monty 9/20/2015 21:28'!
formatBeforeCharacterEscape: aCharacterEscape! !

!XMLWriterFormatter methodsFor: 'formatting' stamp: 'monty 12/1/2014 23:38'!
formatBeforeComment: aCommentWriter! !

!XMLWriterFormatter methodsFor: 'formatting' stamp: 'monty 12/1/2014 23:40'!
formatBeforeDeclaration: aDeclarationWriter! !

!XMLWriterFormatter methodsFor: 'formatting' stamp: 'monty 12/1/2014 23:44'!
formatBeforeDoctypeDeclaration: aDoctypeDeclarationWriter! !

!XMLWriterFormatter methodsFor: 'formatting' stamp: 'monty 12/1/2014 23:38'!
formatBeforeEndTag: aTagWriter! !

!XMLWriterFormatter methodsFor: 'formatting' stamp: 'monty 12/2/2014 00:35'!
formatBeforeEntityReference: anEntityReferenceWriter! !

!XMLWriterFormatter methodsFor: 'formatting' stamp: 'monty 12/1/2014 23:46'!
formatBeforeInternalSubset: aDoctypeDeclarationWriter! !

!XMLWriterFormatter methodsFor: 'formatting' stamp: 'monty 5/8/2015 04:12'!
formatBeforePI: aPIWriter! !

!XMLWriterFormatter methodsFor: 'formatting' stamp: 'monty 9/18/2015 18:53'!
formatBeforeQuotedStringValue: aString! !

!XMLWriterFormatter methodsFor: 'formatting' stamp: 'monty 12/1/2014 23:38'!
formatBeforeStartTag: aTagWriter! !

!XMLWriterFormatter methodsFor: 'formatting' stamp: 'monty 3/24/2015 19:34'!
formatBeforeXMLDeclaration: anXMLDeclarationWriter! !

!XMLWriterFormatter methodsFor: 'private' stamp: 'monty 11/30/2014 03:25'!
indent
	currentIndentLevel := self currentIndentLevel + 1! !

!XMLWriterFormatter methodsFor: 'accessing' stamp: 'monty 11/30/2014 03:16'!
indentString
	^ indentString! !

!XMLWriterFormatter methodsFor: 'accessing' stamp: 'monty 11/30/2014 03:16'!
indentString: aString
	currentIndentLevel := 0.
	indentString := aString.! !

!XMLWriterFormatter methodsFor: 'initialization' stamp: 'monty 9/20/2015 23:24'!
initialize
	super initialize.

	currentIndentLevel := self class defaultIndentLevel.
	indentString := self class defaultIndentString.! !

!XMLWriterFormatter methodsFor: 'private' stamp: 'monty 11/30/2014 03:25'!
outdent
	currentIndentLevel := self currentIndentLevel - 1! !

!XMLWriterFormatter methodsFor: 'private' stamp: 'monty 7/4/2016 06:31'!
writeIndent
	"use to:do: over timesRepeat: for speed"
	1 to: self currentIndentLevel do: [:i |
		self writer nextPutAll: self indentString]! !

!XMLWriterFormatter methodsFor: 'private' stamp: 'monty 12/13/2014 21:06'!
writeLineBreak
	self writer atBeginning
		ifFalse: [
			self writer nextPutAll: self writer lineBreak.
			(self currentIndentLevel > 0
				and: [self indentString notEmpty])
				ifTrue: [self writeIndent]]! !

!XMLWriterFormatter methodsFor: 'accessing' stamp: 'monty 11/30/2014 02:44'!
writer
	^ writer! !

!XMLWriterFormatter methodsFor: 'accessing' stamp: 'monty 9/21/2015 01:01'!
writer: aWriter
	writer := aWriter! !

!BlockClosure methodsFor: '*XML-Writer-Core' stamp: 'monty 4/20/2016 07:47'!
embedInXMLMarkupWriter: aMarkupWriter
	self cull: aMarkupWriter writer! !

XMLWriter initialize!
